@@ -18,7 +18,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1998 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -108,8 +108,15 @@
 /*
   Include declarations.
 */
+#if !defined(macintosh)
+#include "magick/magick.h"
+#include "magick/defines.h"
+#include "magick/xwindows.h"
+#else
 #include "magick.h"
-#include "version.h"
+#include "defines.h"
+#include "xwindows.h"
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -122,11 +129,11 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function Usage displays the program command syntax.
+%  Method Usage displays the program command syntax.
 %
-%  The format of the Usage routine is:
+%  The format of the Usage method is:
 %
-%      Usage(client_name)
+%      void Usage(const char *client_name)
 %
 %  A description of each parameter follows:
 %
@@ -137,10 +144,10 @@
 */
 static void Usage(const char *client_name)
 {
-  char
+  const char
     **p;
 
-  static char
+  static const char
     *buttons[]=
     {
       "Press any button to map or unmap the Command widget",
@@ -174,13 +181,14 @@ static void Usage(const char *client_name)
       (char *) NULL
     };
 
-  (void) printf("Version: %s\n",Version);
-  (void) printf("Copyright: %s\n\n",Copyright);
+  (void) printf("Version: %.1024s\n",MagickVersion);
+  (void) printf("Copyright: %.1024s\n\n",MagickCopyright);
   (void) printf(
-    "Usage: %s [-options ...] file [ [-options ...] file ...]\n",client_name);
+    "Usage: %.1024s [-options ...] file [ [-options ...] file ...]\n",
+    client_name);
   (void) printf("\nWhere options include: \n");
   for (p=options; *p != (char *) NULL; p++)
-    (void) printf("  %s\n",*p);
+    (void) printf("  %.1024s\n",*p);
   (void) printf(
     "\nIn addition to those listed above, you can specify these standard X\n");
   (void) printf(
@@ -199,7 +207,7 @@ static void Usage(const char *client_name)
   (void) printf("'-' for standard input or output.\n");
   (void) printf("\nButtons: \n");
   for (p=buttons; *p != (char *) NULL; p++)
-    (void) printf("  %s\n",*p);
+    (void) printf("  %.1024s\n",*p);
   Exit(0);
 }
 
@@ -234,6 +242,7 @@ int WINAPI WinMain(HINSTANCE instance,HINSTANCE last,LPSTR command,int state)
 
 int main(int argc,char **argv)
 {
+#if defined(HasX11)
   char
     *client_name,
     *option,
@@ -243,6 +252,9 @@ int main(int argc,char **argv)
   Display
     *display;
 
+  double
+    sans;
+
   Image
     *image,
     *loaded_image,
@@ -271,14 +283,15 @@ int main(int argc,char **argv)
     resource_database;
 
   /*
-    Set defaults.
+    Initialize command line arguments.
   */
-#if defined(WIN32)
-  SetErrorHandler(NTErrorHandler);
-  SetWarningHandler(NTWarningHandler);
-#endif
+  SetNotifyHandlers;
   ReadCommandlLine(argc,&argv);
   client_name=SetClientName(*argv);
+  (void) ExpandFilenames(&argc,&argv);
+  /*
+    Set defaults.
+  */
   display=(Display *) NULL;
   first_scene=0;
   image=(Image *) NULL;
@@ -286,7 +299,6 @@ int main(int argc,char **argv)
   /*
     Check for server name specified on the command line.
   */
-  ExpandFilenames(&argc,&argv);
   server_name=(char *) NULL;
   for (i=1; i < argc; i++)
   {
@@ -296,7 +308,7 @@ int main(int argc,char **argv)
     option=argv[i];
     if ((Extent(option) == 1) || ((*option != '-') && (*option != '+')))
       continue;
-    if (strcmp("display",option+1) == 0)
+    if (Latin1Compare("display",option+1) == 0)
       {
         /*
           User specified server name.
@@ -320,13 +332,16 @@ int main(int argc,char **argv)
   XSetErrorHandler(XError);
   resource_database=XGetResourceDatabase(display,client_name);
   XGetResourceInfo(resource_database,client_name,&resource_info);
+  image_info=resource_info.image_info;
+  image_info->insert_backdrops=True;
+  image_info->coalesce_frames=True;
+  quantize_info=resource_info.quantize_info;
   resource_info.delay=0;
   resource_info.pause=0;
   resource_value=
-    XGetResourceInstance(resource_database,client_name,"delay","6");
+    XGetResourceInstance(resource_database,client_name,"delay","0");
   (void) XParseGeometry(resource_value,&x,&x,&resource_info.delay,
     &resource_info.pause);
-  image_info=(&resource_info.image_info);
   image_info->density=
     XGetResourceInstance(resource_database,client_name,"density",(char *) NULL);
   if (image_info->density == (char *) NULL)
@@ -347,7 +362,9 @@ int main(int argc,char **argv)
   resource_value=
     XGetResourceInstance(resource_database,client_name,"verbose","False");
   image_info->verbose=IsTrue(resource_value);
-  quantize_info=(&resource_info.quantize_info);
+  resource_value=
+    XGetResourceInstance(resource_database,client_name,"dither","True");
+  quantize_info->dither=IsTrue(resource_value);
   /*
     Parse command line.
   */
@@ -381,7 +398,8 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing color",option);
-                  resource_info.background_color=argv[i];
+                  (void) CloneString(&resource_info.background_color,argv[i]);
+                  (void) CloneString(&image_info->background_color,argv[i]);
                 }
               break;
             }
@@ -393,7 +411,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing color",option);
-                  resource_info.border_color=argv[i];
+                  (void) CloneString(&resource_info.border_color,argv[i]);
                 }
               break;
             }
@@ -467,12 +485,16 @@ int main(int argc,char **argv)
                     quantize_info->colorspace=OHTAColorspace;
                   if (Latin1Compare("rgb",option) == 0)
                     quantize_info->colorspace=RGBColorspace;
+                  if (Latin1Compare("srgb",option) == 0)
+                    quantize_info->colorspace=sRGBColorspace;
                   if (Latin1Compare("transparent",option) == 0)
                     quantize_info->colorspace=TransparentColorspace;
                   if (Latin1Compare("xyz",option) == 0)
                     quantize_info->colorspace=XYZColorspace;
                   if (Latin1Compare("ycbcr",option) == 0)
                     quantize_info->colorspace=YCbCrColorspace;
+                  if (Latin1Compare("ycc",option) == 0)
+                    quantize_info->colorspace=YCCColorspace;
                   if (Latin1Compare("yiq",option) == 0)
                     quantize_info->colorspace=YIQColorspace;
                   if (Latin1Compare("ypbpr",option) == 0)
@@ -526,11 +548,11 @@ int main(int argc,char **argv)
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  image_info->density=argv[i];
+                  (void) CloneString(&image_info->density,argv[i]);
                 }
               break;
             }
-          if (strcmp("display",option+1) == 0)
+          if (Latin1Compare("display",option+1) == 0)
             {
               image_info->server_name=(char *) NULL;
               if (*option == '-')
@@ -538,7 +560,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing server name",option);
-                  image_info->server_name=argv[i];
+                  (void) CloneString(&image_info->server_name,argv[i]);
                 }
               break;
             }
@@ -560,9 +582,11 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing font name",option);
-                  image_info->font=argv[i];
+                  (void) CloneString(&image_info->font,argv[i]);
                 }
-              resource_info.font=image_info->font;
+              if ((image_info->font == (char *) NULL) ||
+                  (*image_info->font != '@'))
+                (void) CloneString(&resource_info.font,image_info->font);
               break;
             }
           if (strncmp("foreground",option+1,3) == 0)
@@ -573,7 +597,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing foreground",option);
-                  resource_info.foreground_color=argv[i];
+                  (void) CloneString(&resource_info.foreground_color,argv[i]);
                 }
               break;
             }
@@ -585,7 +609,7 @@ int main(int argc,char **argv)
           if (strncmp("gamma",option+1,2) == 0)
             {
               i++;
-              if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+              if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                 MagickError(OptionError,"Missing value",option);
               break;
             }
@@ -597,7 +621,7 @@ int main(int argc,char **argv)
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  resource_info.image_geometry=argv[i];
+                  (void) CloneString(&resource_info.image_geometry,argv[i]);
                 }
               break;
             }
@@ -624,7 +648,7 @@ int main(int argc,char **argv)
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  resource_info.icon_geometry=argv[i];
+                  (void) CloneString(&resource_info.icon_geometry,argv[i]);
                 }
               break;
             }
@@ -670,11 +694,11 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing map type",option);
-                  resource_info.map_type=argv[i];
+                  (void) CloneString(&resource_info.map_type,argv[i]);
                 }
               break;
             }
-          if (strcmp("matte",option+1) == 0)
+          if (Latin1Compare("matte",option+1) == 0)
             break;
           if (strncmp("mattecolor",option+1,6) == 0)
             {
@@ -684,7 +708,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing color",option);
-                  resource_info.matte_color=argv[i];
+                  (void) CloneString(&resource_info.matte_color,argv[i]);
                 }
               break;
             }
@@ -710,7 +734,7 @@ int main(int argc,char **argv)
               i++;
               if (i == argc)
                 MagickError(OptionError,"Missing name",option);
-              resource_info.name=argv[i];
+              (void) CloneString(&resource_info.name,argv[i]);
             }
           break;
         }
@@ -718,28 +742,10 @@ int main(int argc,char **argv)
         {
           if (strncmp("remote",option+1,3) == 0)
             {
-              Atom
-                remote_atom;
-
-              Window
-                remote_window,
-                root_window;
-
-              root_window=XRootWindow(display,XDefaultScreen(display));
-              remote_atom=XInternAtom(display,"IM_PROTOCOLS",False);
-              remote_window=XWindowByProperty(display,root_window,remote_atom);
-              if (remote_window == (Window) NULL)
-                break;
               i++;
               if (i == argc)
                 MagickError(OptionError,"Missing command",option);
-              /*
-                Send remote command.
-              */
-              remote_atom=XInternAtom(display,"IM_REMOTE_COMMAND",False);
-              XChangeProperty(display,remote_window,remote_atom,XA_STRING,8,
-                PropModeReplace,(unsigned char *) argv[i],Extent(argv[i]));
-              XSync(display,False);
+              XRemoteCommand(display,resource_info.window_id,argv[i]);
               Exit(0);
             }
           if (strncmp("rotate",option+1,3) == 0)
@@ -785,7 +791,7 @@ int main(int argc,char **argv)
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  image_info->size=argv[i];
+                  (void) CloneString(&image_info->size,argv[i]);
                 }
               break;
             }
@@ -802,7 +808,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing font name",option);
-                  resource_info.text_font=argv[i];
+                  (void) CloneString(&resource_info.text_font,argv[i]);
                 }
               break;
             }
@@ -814,7 +820,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing title",option);
-                  resource_info.title=argv[i];
+                  (void) CloneString(&resource_info.title,argv[i]);
                 }
               break;
             }
@@ -848,7 +854,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing visual class",option);
-                  resource_info.visual_type=argv[i];
+                  (void) CloneString(&resource_info.visual_type,argv[i]);
                 }
               break;
             }
@@ -866,7 +872,7 @@ int main(int argc,char **argv)
                   if (i == argc)
                     MagickError(OptionError,"Missing id, name, or 'root'",
                       option);
-                  resource_info.window_id=argv[i];
+                  (void) CloneString(&resource_info.window_id,argv[i]);
                 }
               break;
             }
@@ -903,9 +909,9 @@ int main(int argc,char **argv)
               /*
                 Form filename for multi-part images.
               */
-              (void) sprintf(filename,image_info->filename,scene);
-              if (strcmp(filename,image_info->filename) == 0)
-                (void) sprintf(filename,"%s[%u]",image_info->filename,scene);
+              FormatString(filename,image_info->filename,scene);
+              if (Latin1Compare(filename,image_info->filename) == 0)
+                FormatString(filename,"%.1024s[%u]",image_info->filename,scene);
               (void) strcpy(image_info->filename,filename);
             }
           (void) strcpy(image_info->magick,"MIFF");
@@ -913,10 +919,12 @@ int main(int argc,char **argv)
           image_info->dither=quantize_info->dither;
           next_image=ReadImage(image_info);
           if (next_image == (Image *) NULL)
-            if (*option == '-')
-              break;
-            else
-              continue;
+            {
+              if (*option == '-')
+                break;
+              else
+                continue;
+            }
           MogrifyImages(image_info,i,argv,&next_image);
           if (image == (Image *) NULL)
             image=next_image;
@@ -953,6 +961,8 @@ int main(int argc,char **argv)
         DestroyImages(image);
       }
     }
+  DestroyDelegateInfo();
   Exit(0);
+#endif
   return(False);
 }
