@@ -18,7 +18,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1998 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -71,7 +71,7 @@
 %    -colors value      preferred number of colors in the image
 %    -colorspace type   alternate image colorspace
 %    -comment string    annotate image with comment",
-%    -compress type     None, JPEG, LZW, RunlengthEncoded, or Zip
+%    -compress type     type of image compression
 %    -contrast          enhance or reduce the image contrast
 %    -crop geometry     preferred size and location of the cropped image
 %    -delay value       display the next image after pausing
@@ -82,6 +82,7 @@
 %    -dither            apply Floyd/Steinberg error diffusion to image
 %    -edge factor       apply a filter to detect edges in the image
 %    -enhance           apply a digital filter to enhance a noisy image
+%    -filter type       use this filter when resizing an image
 %    -flip              flip image in the vertical direction
 %    -flop              flop image in the horizontal direction
 %    -frame geometry    surround image with an ornamental border
@@ -93,7 +94,7 @@
 %    -map type          display image using this Standard Colormap
 %    -matte             store matte channel if the image has one
 %    -monochrome        transform image to black and white
-%    -negate            apply color inversion to image
+%    -negate            replace every pixel with its complementary color
 %    -page geometry     size and location of an image canvas
 %    -quality value     JPEG/MIFF/PNG compression level
 %    -raise value       lighten/darken image edges to create a 3-D effect
@@ -136,8 +137,15 @@
 /*
   Include declarations.
 */
+#if !defined(macintosh)
+#include "magick/magick.h"
+#include "magick/defines.h"
+#include "magick/xwindows.h"
+#else
 #include "magick.h"
-#include "version.h"
+#include "defines.h"
+#include "xwindows.h"
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -150,11 +158,11 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function Usage displays the program command syntax.
+%  Method Usage displays the program command syntax.
 %
-%  The format of the Usage routine is:
+%  The format of the Usage method is:
 %
-%      Usage(client_name)
+%      void Usage(const char *client_name)
 %
 %  A description of each parameter follows:
 %
@@ -164,10 +172,10 @@
 */
 static void Usage(const char *client_name)
 {
-  char
+  const char
     **p;
 
-  static char
+  static const char
     *buttons[]=
     {
       "1    press to map or unmap the Command widget",
@@ -183,7 +191,7 @@ static void Usage(const char *client_name)
       "-colors value      preferred number of colors in the image",
       "-colorspace type   alternate image colorspace",
       "-comment string    annotate image with comment",
-      "-compress type     None, JPEG, LZW, RunlengthEncoded, or Zip",
+      "-compress type     type of image compression",
       "-contrast          enhance or reduce the image contrast",
       "-crop geometry     preferred size and location of the cropped image",
       "-delay value       display the next image after pausing",
@@ -194,6 +202,7 @@ static void Usage(const char *client_name)
       "-dither            apply Floyd/Steinberg error diffusion to image",
       "-edge factor       apply a filter to detect edges in the image",
       "-enhance           apply a digital filter to enhance a noisy image",
+      "-filter type       use this filter when resizing an image",
       "-flip              flip image in the vertical direction",
       "-flop              flop image in the horizontal direction",
       "-frame geometry    surround image with an ornamental border",
@@ -205,7 +214,7 @@ static void Usage(const char *client_name)
       "-map type          display image using this Standard Colormap",
       "-matte             store matte channel if the image has one",
       "-monochrome        transform image to black and white",
-      "-negate            apply color inversion to image",
+      "-negate            replace every pixel with its complementary color ",
       "-page geometry     size and location of an image canvas",
       "-quality value     JPEG/MIFF/PNG compression level",
       "-raise value       lighten/darken image edges to create a 3-D effect",
@@ -228,13 +237,14 @@ static void Usage(const char *client_name)
       (char *) NULL
     };
 
-  (void) printf("Version: %s\n",Version);
-  (void) printf("Copyright: %s\n\n",Copyright);
+  (void) printf("Version: %.1024s\n",MagickVersion);
+  (void) printf("Copyright: %.1024s\n\n",MagickCopyright);
   (void) printf(
-    "Usage: %s [-options ...] file [ [-options ...] file ...]\n",client_name);
+    "Usage: %.1024s [-options ...] file [ [-options ...] file ...]\n",
+    client_name);
   (void) printf("\nWhere options include: \n");
   for (p=options; *p != (char *) NULL; p++)
-    (void) printf("  %s\n",*p);
+    (void) printf("  %.1024s\n",*p);
   (void) printf(
     "\nIn addition to those listed above, you can specify these standard X\n");
   (void) printf(
@@ -253,7 +263,7 @@ static void Usage(const char *client_name)
   (void) printf("'-' for standard input or output.\n");
   (void) printf("\nButtons: \n");
   for (p=buttons; *p != (char *) NULL; p++)
-    (void) printf("  %s\n",*p);
+    (void) printf("  %.1024s\n",*p);
   Exit(0);
 }
 
@@ -288,6 +298,7 @@ int WINAPI WinMain(HINSTANCE instance,HINSTANCE last,LPSTR command,int state)
 
 int main(int argc,char **argv)
 {
+#if defined(HasX11)
   char
     *client_name,
     *option,
@@ -297,6 +308,9 @@ int main(int argc,char **argv)
   Display
     *display;
 
+  double
+    sans;
+
   Image
     *image,
     *next_image;
@@ -305,6 +319,7 @@ int main(int argc,char **argv)
     *image_info;
 
   int
+    image_number,
     status,
     x;
 
@@ -318,7 +333,7 @@ int main(int argc,char **argv)
   unsigned int
     first_scene,
     *image_marker,
-    image_number,
+    last_image,
     last_scene,
     scene;
 
@@ -332,31 +347,31 @@ int main(int argc,char **argv)
     resource_database;
 
   /*
-    Set defaults.
+    Initialize command line arguments.
   */
-#if defined(WIN32)
-  SetErrorHandler(NTErrorHandler);
-  SetWarningHandler(NTWarningHandler);
-#endif
+  SetNotifyHandlers;
   ReadCommandlLine(argc,&argv);
   client_name=SetClientName(*argv);
+  (void) ExpandFilenames(&argc,&argv);
+  /*
+    Set defaults.
+  */
   display=(Display *) NULL;
   first_scene=0;
-  image=(Image *) NULL;
-  image_marker=(unsigned int *) malloc((argc+1)*sizeof(unsigned int));
+  image_number=0;
+  last_image=0;
+  last_scene=0;
+  image_marker=(unsigned int *) AllocateMemory((argc+1)*sizeof(unsigned int));
   if (image_marker == (unsigned int *) NULL)
     MagickError(ResourceLimitError,"Unable to display image",
       "Memory allocation failed");
   for (i=0; i <= argc; i++)
     image_marker[i]=argc;
-  image_number=0;
-  last_scene=0;
   resource_database=(XrmDatabase) NULL;
-  state=DefaultState;
+  state=0;
   /*
     Check for server name specified on the command line.
   */
-  ExpandFilenames(&argc,&argv);
   server_name=(char *) NULL;
   for (i=1; i < argc; i++)
   {
@@ -366,7 +381,7 @@ int main(int argc,char **argv)
     option=argv[i];
     if ((Extent(option) == 1) || ((*option != '-') && (*option != '+')))
       continue;
-    if (strcmp("display",option+1) == 0)
+    if (Latin1Compare("display",option+1) == 0)
       {
         /*
           User specified server name.
@@ -390,7 +405,9 @@ int main(int argc,char **argv)
   XSetErrorHandler(XError);
   resource_database=XGetResourceDatabase(display,client_name);
   XGetResourceInfo(resource_database,client_name,&resource_info);
-  image_info=(&resource_info.image_info);
+  image_info=resource_info.image_info;
+  image_info->coalesce_frames=True;
+  quantize_info=resource_info.quantize_info;
   image_info->density=
     XGetResourceInstance(resource_database,client_name,"density",(char *) NULL);
   if (image_info->density == (char *) NULL)
@@ -416,7 +433,9 @@ int main(int argc,char **argv)
   resource_value=
     XGetResourceInstance(resource_database,client_name,"verbose","False");
   image_info->verbose=IsTrue(resource_value);
-  quantize_info=(&resource_info.quantize_info);
+  resource_value=
+    XGetResourceInstance(resource_database,client_name,"dither","True");
+  quantize_info->dither=IsTrue(resource_value);
   /*
     Parse command line.
   */
@@ -451,11 +470,12 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing color",option);
-                  resource_info.background_color=argv[i];
+                  (void) CloneString(&resource_info.background_color,argv[i]);
+                  (void) CloneString(&image_info->background_color,argv[i]);
                 }
               break;
             }
-          if (strcmp("border",option+1) == 0)
+          if (Latin1Compare("border",option+1) == 0)
             {
               if (*option == '-')
                 {
@@ -473,7 +493,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing color",option);
-                  resource_info.border_color=argv[i];
+                  (void) CloneString(&resource_info.border_color,argv[i]);
                 }
               break;
             }
@@ -547,12 +567,16 @@ int main(int argc,char **argv)
                     quantize_info->colorspace=OHTAColorspace;
                   if (Latin1Compare("rgb",option) == 0)
                     quantize_info->colorspace=RGBColorspace;
+                  if (Latin1Compare("srgb",option) == 0)
+                    quantize_info->colorspace=sRGBColorspace;
                   if (Latin1Compare("transparent",option) == 0)
                     quantize_info->colorspace=TransparentColorspace;
                   if (Latin1Compare("xyz",option) == 0)
                     quantize_info->colorspace=XYZColorspace;
                   if (Latin1Compare("ycbcr",option) == 0)
                     quantize_info->colorspace=YCbCrColorspace;
+                  if (Latin1Compare("ycc",option) == 0)
+                    quantize_info->colorspace=YCCColorspace;
                   if (Latin1Compare("yiq",option) == 0)
                     quantize_info->colorspace=YIQColorspace;
                   if (Latin1Compare("ypbpr",option) == 0)
@@ -586,6 +610,12 @@ int main(int argc,char **argv)
                   image_info->compression=UndefinedCompression;
                   if (Latin1Compare("None",option) == 0)
                     image_info->compression=NoCompression;
+                  if (Latin1Compare("BZip",option) == 0)
+                    image_info->compression=BZipCompression;
+                  if (Latin1Compare("Fax",option) == 0)
+                    image_info->compression=FaxCompression;
+                  if (Latin1Compare("Group4",option) == 0)
+                    image_info->compression=Group4Compression;
                   if (Latin1Compare("JPEG",option) == 0)
                     image_info->compression=JPEGCompression;
                   if (Latin1Compare("LZW",option) == 0)
@@ -641,13 +671,13 @@ int main(int argc,char **argv)
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  image_info->density=argv[i];
+                  (void) CloneString(&image_info->density,argv[i]);
                 }
               break;
             }
           if (strncmp("despeckle",option+1,3) == 0)
             break;
-          if (strcmp("display",option+1) == 0)
+          if (Latin1Compare("display",option+1) == 0)
             {
               image_info->server_name=(char *) NULL;
               if (*option == '-')
@@ -686,7 +716,7 @@ int main(int argc,char **argv)
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing factor",option);
                 }
               break;
@@ -698,6 +728,53 @@ int main(int argc,char **argv)
         }
         case 'f':
         {
+          if (strncmp("filter",option+1,3) == 0)
+            {
+              if (*option == '-')
+                {
+                  FilterType
+                    filter;
+
+                  i++;
+                  if (i == argc)
+                    MagickError(OptionError,"Missing type",option);
+                  option=argv[i];
+                  filter=UndefinedFilter;
+                  if (Latin1Compare("Point",option) == 0)
+                    filter=PointFilter;
+                  if (Latin1Compare("Box",option) == 0)
+                    filter=BoxFilter;
+                  if (Latin1Compare("Triangle",option) == 0)
+                    filter=TriangleFilter;
+                  if (Latin1Compare("Hermite",option) == 0)
+                    filter=HermiteFilter;
+                  if (Latin1Compare("Hanning",option) == 0)
+                    filter=HanningFilter;
+                  if (Latin1Compare("Hamming",option) == 0)
+                    filter=HammingFilter;
+                  if (Latin1Compare("Blackman",option) == 0)
+                    filter=BlackmanFilter;
+                  if (Latin1Compare("Gaussian",option) == 0)
+                    filter=GaussianFilter;
+                  if (Latin1Compare("Quadratic",option) == 0)
+                    filter=QuadraticFilter;
+                  if (Latin1Compare("Cubic",option) == 0)
+                    filter=CubicFilter;
+                  if (Latin1Compare("Catrom",option) == 0)
+                    filter=CatromFilter;
+                  if (Latin1Compare("Mitchell",option) == 0)
+                    filter=MitchellFilter;
+                  if (Latin1Compare("Lanczos",option) == 0)
+                    filter=LanczosFilter;
+                  if (Latin1Compare("Bessel",option) == 0)
+                    filter=BesselFilter;
+                  if (Latin1Compare("Sinc",option) == 0)
+                    filter=SincFilter;
+                  if (filter == UndefinedFilter)
+                    MagickError(OptionError,"Invalid filter type",option);
+                }
+              break;
+            }
           if (strncmp("flip",option+1,3) == 0)
             break;
           if (strncmp("flop",option+1,3) == 0)
@@ -712,7 +789,9 @@ int main(int argc,char **argv)
                     MagickError(OptionError,"Missing font name",option);
                   image_info->font=argv[i];
                 }
-              resource_info.font=image_info->font;
+              if ((image_info->font == (char *) NULL) ||
+                  (*image_info->font != '@'))
+                resource_info.font=image_info->font;
               break;
             }
           if ((strncmp("foreground",option+1,3) == 0) ||
@@ -746,7 +825,7 @@ int main(int argc,char **argv)
           if (strncmp("gamma",option+1,2) == 0)
             {
               i++;
-              if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+              if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                 MagickError(OptionError,"Missing value",option);
               break;
             }
@@ -865,7 +944,7 @@ int main(int argc,char **argv)
                 }
               break;
             }
-          if (strcmp("matte",option+1) == 0)
+          if (Latin1Compare("matte",option+1) == 0)
             break;
           if (strncmp("mattecolor",option+1,6) == 0)
             {
@@ -933,7 +1012,7 @@ int main(int argc,char **argv)
         {
           if (strncmp("quality",option+1,2) == 0)
             {
-              image_info->quality=atoi(DefaultImageQuality);
+              image_info->quality=75;
               if (*option == '-')
                 {
                   i++;
@@ -957,28 +1036,10 @@ int main(int argc,char **argv)
             }
           if (strncmp("remote",option+1,3) == 0)
             {
-              Atom
-                remote_atom;
-
-              Window
-                remote_window,
-                root_window;
-
-              root_window=XRootWindow(display,XDefaultScreen(display));
-              remote_atom=XInternAtom(display,"IM_PROTOCOLS",False);
-              remote_window=XWindowByProperty(display,root_window,remote_atom);
-              if (remote_window == (Window) NULL)
-                break;
               i++;
               if (i == argc)
                 MagickError(OptionError,"Missing command",option);
-              /*
-                Send remote command.
-              */
-              remote_atom=XInternAtom(display,"IM_REMOTE_COMMAND",False);
-              XChangeProperty(display,remote_window,remote_atom,XA_STRING,8,
-                PropModeReplace,(unsigned char *) argv[i],Extent(argv[i]));
-              XSync(display,False);
+              XRemoteCommand(display,resource_info.window_id,argv[i]);
               Exit(0);
             }
           if (strncmp("roll",option+1,3) == 0)
@@ -1036,7 +1097,7 @@ int main(int argc,char **argv)
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing threshold",option);
                 }
               break;
@@ -1046,7 +1107,7 @@ int main(int argc,char **argv)
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing factor",option);
                 }
               break;
@@ -1073,7 +1134,7 @@ int main(int argc,char **argv)
         }
         case 't':
         {
-          if (strncmp("text_font",option+1,3) == 0)
+          if (strncmp("text_font",option+1,5) == 0)
             {
               resource_info.text_font=(char *) NULL;
               if (*option == '-')
@@ -1170,7 +1231,7 @@ int main(int argc,char **argv)
         }
         case 'w':
         {
-          if (strcmp("window",option+1) == 0)
+          if (Latin1Compare("window",option+1) == 0)
             {
               resource_info.window_id=(char *) NULL;
               if (*option == '-')
@@ -1210,7 +1271,7 @@ int main(int argc,char **argv)
                       char
                         answer[2];
 
-                      (void) fprintf(stderr,"Overwrite %s? ",
+                      (void) fprintf(stderr,"Overwrite %.1024s? ",
                         resource_info.write_filename);
                       (void) fgets(answer,sizeof(answer),stdin);
                       if (!((*answer == 'y') || (*answer == 'Y')))
@@ -1252,9 +1313,9 @@ int main(int argc,char **argv)
               /*
                 Form filename for multi-part images.
               */
-              (void) sprintf(filename,image_info->filename,scene);
-              if (strcmp(filename,image_info->filename) == 0)
-                (void) sprintf(filename,"%s.%u",image_info->filename,scene);
+              FormatString(filename,image_info->filename,scene);
+              if (Latin1Compare(filename,image_info->filename) == 0)
+                FormatString(filename,"%.1024s.%u",image_info->filename,scene);
               (void) strcpy(image_info->filename,filename);
             }
           (void) strcpy(image_info->magick,"MIFF");
@@ -1262,13 +1323,15 @@ int main(int argc,char **argv)
           image_info->dither=quantize_info->dither;
           image=ReadImage(image_info);
           if (image == (Image *) NULL)
-            if ((i < (argc-1)) || (scene < last_scene))
-              continue;
-            else
-              {
-                state|=ExitState;
-                break;
-              }
+            {
+              if ((i < (argc-1)) || (scene < last_scene))
+                continue;
+              else
+                {
+                  state|=ExitState;
+                  break;
+                }
+            }
           do
           {
             /*
@@ -1343,10 +1406,11 @@ int main(int argc,char **argv)
                   Write image.
                 */
                 (void) strcpy(image->filename,resource_info.write_filename);
+                SetImageInfo(image_info,True);
                 (void) WriteImage(image_info,image);
               }
             if (image_info->verbose)
-              DescribeImage(image,stderr,False);
+              DescribeImage(image,stdout,False);
             /*
               Proceed to next/previous image.
             */
@@ -1373,7 +1437,10 @@ int main(int argc,char **argv)
           */
           DestroyImages(image);
           if (!(state & FormerImageState))
-            image_marker[i]=image_number++;
+            {
+              last_image=image_number;
+              image_marker[i]=image_number++;
+            }
           else
             {
               /*
@@ -1382,8 +1449,7 @@ int main(int argc,char **argv)
               for (i--; i > 0; i--)
                 if (image_marker[i] == (image_number-2))
                   break;
-              if (image_number != 0)
-                image_number--;
+              image_number--;
             }
           if (state & ExitState)
             break;
@@ -1392,19 +1458,33 @@ int main(int argc,char **argv)
     /*
       Determine if we should proceed to the first image.
     */
-    if ((i == (argc-1)) || (argc == 1))
-      if (!(state & ExitState))
-        if (resource_info.window_id == (char *) NULL)
+    if (image_number < 0)
+      {
+        if (state & FormerImageState)
           {
-            i=0;
-            image_number=0;
+            for (i=1; i < (argc-2); i++)
+              if (image_marker[i] == last_image)
+                break;
+            image_number=image_marker[i]+1;
           }
+        continue;
+      }
+    if (image_number > 0)
+      if ((i == (argc-1)) || (argc == 1))
+        if (!(state & ExitState))
+          if (resource_info.window_id == (char *) NULL)
+            {
+              i=0;
+              image_number=0;
+            }
   }
   if (state & RetainColorsState)
     {
       XRetainWindowColors(display,XRootWindow(display,XDefaultScreen(display)));
       XSync(display,False);
     }
+  DestroyDelegateInfo();
   Exit(0);
+#endif
   return(False);
 }
