@@ -1,14 +1,32 @@
+/*
+  Methods to Interactively Display and Edit an Image.
+*/
+#ifndef _DISPLAY_H
+#define _DISPLAY_H
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
 /*
   Define declarations.
 */
 #define MaxColors  Min(windows->visual_info->colormap_size,256)
-#define MinPanSize  96
-#define RoiDelta  8
 
 /*
-  Static declarations.
+  Constant declarations.
 */
-static char
+static const unsigned char
+  HighlightBitmap[8] =
+  {
+    0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55
+  },
+  ShadowBitmap[8] =
+  {
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
+  };
+
+const char
   *NoiseTypes[] =
   {
     "Uniform",
@@ -58,22 +76,23 @@ static char
     "Threshold",
     "Edge Detect",
     "Spread",
+    "Solarize",
     "Shade",
     "Raise",
     "Segment",
-    "Solarize",
     "Swirl",
     "Implode",
     "Wave",
     "Oil Paint",
     "Charcoal Drawing",
+    "JPEG",
     (char *) NULL
   };
 
 /*
   Help widget declarations.
 */
-static char
+const char
   *ImageAnnotateHelp[]=
   {
     "In annotate mode, the Command widget has these options:",
@@ -203,6 +222,12 @@ static char
   {
     "In color edit mode, the Command widget has these options:",
     "",
+    "    Method",
+    "      point",
+    "      replace",
+    "      floodfill",
+    "      filltoborder",
+    "      reset",
     "    Pixel Color",
     "      black",
     "      blue",
@@ -214,44 +239,46 @@ static char
     "      yellow",
     "      white",
     "      Browser...",
-    "    Method",
-    "      point",
-    "      replace",
-    "      floodfill",
-    "      reset",
-    "    Delta",
+    "    Border Color",
+    "      black",
+    "      blue",
+    "      cyan",
+    "      green",
+    "      gray",
+    "      red",
+    "      magenta",
+    "      yellow",
+    "      white",
+    "      Browser...",
+    "    Fuzz",
     "      0",
-    "      1",
     "      2",
     "      4",
     "      8",
     "      16",
-    "      32",
-    "      Browser...",
+    "      Dialog...",
     "    Undo",
     "    Help",
     "    Dismiss",
     "",
-    "Choose a pixel color from the Pixel Color sub-menu.",
-    "Additional pixel colors can be specified with the color",
-    "browser.  You can change the menu colors by setting the X",
-    "resources pen1 through pen9.",
-    "",
-    "Next, choose a color editing method from the Method sub-menu",
+    "Choose a color editing method from the Method sub-menu",
     "of the Command widget.  The point method recolors any pixel",
     "selected with the pointer until the button is released.  The",
     "replace method recolors any pixel that matches the color of",
     "the pixel you select with a button press.  Floodfill recolors",
     "any pixel that matches the color of the pixel you select with",
-    "a button press and is a neighbor.  Finally reset changes the",
-    "entire image to the designated color.",
+    "a button press and is a neighbor.  Whereas filltoborder recolors",
+    "any neighbor pixel that is not the border color.  Finally reset",
+    "changes the entire image to the designated color.",
+    "",
+    "Next, choose a pixel color from the Pixel Color sub-menu.",
+    "Additional pixel colors can be specified with the color",
+    "browser.  You can change the menu colors by setting the X",
+    "resources pen1 through pen9.",
     "",
     "Now press button 1 to select a pixel within the image window",
     "to change its color.  Additional pixels may be recolored as",
-    "prescribed by the method you choose.  You can recolor additional",
-    "pixels by increasing the Delta value.  The Delta value is first",
-    "added then subtracted from the red, green, and blue of the target",
-    "color.  Any pixels within the range is also recolored.",
+    "prescribed by the method you choose.",
     "",
     "If the Magnify widget is mapped, it can be helpful in positioning",
     "your pointer within the image (refer to button 2).",
@@ -513,6 +540,8 @@ static char
     "      line",
     "      rectangle",
     "      fill rectangle",
+    "      circle",
+    "      fill circle",
     "      ellipse",
     "      fill ellipse",
     "      polygon",
@@ -580,7 +609,7 @@ static char
     "starting point of the line and release the button.",
     (char *) NULL,
   },
-  *ImageMagickHelp[]=
+  *DisplayHelp[]=
   {
     "BUTTONS",
     "  The effects of each button press is described below.  Three",
@@ -923,16 +952,26 @@ static char
     "      point",
     "      replace",
     "      floodfill",
+    "      filltoborder",
     "      reset",
-    "    Delta",
+    "    Border Color",
+    "      black",
+    "      blue",
+    "      cyan",
+    "      green",
+    "      gray",
+    "      red",
+    "      magenta",
+    "      yellow",
+    "      white",
+    "      Browser...",
+    "    Fuzz",
     "      0",
-    "      1",
     "      2",
     "      4",
     "      8",
     "      16",
-    "      32",
-    "      Browser...",
+    "      Dialog...",
     "    Matte",
     "    Undo",
     "    Help",
@@ -945,19 +984,16 @@ static char
     "any pixel that matches the color of the pixel you select with",
     "a button press.  Floodfill changes the matte value of any pixel",
     "that matches the color of the pixel you select with a button",
-    "press and is a neighbor.  Finally reset changes the entire image",
-    "to the designated matte value.",
+    "press and is a neighbor.  Whereas filltoborder changes the matte",
+    "value any neighbor pixel that is not the border color.  Finally",
+    "reset changes the entire image to the designated matte value.",
     "",
     "Choose Matte Value and a dialog appears requesting a matte",
     "value.  Enter a value between 0 and 255.  This value is",
     "assigned as the matte value of the selected pixel or pixels.",
     "",
     "Now, press any button to select a pixel within the image",
-    "window to change its matte value.  You can change the matte",
-    "value of additional pixels by increasing the Delta value.  The",
-    "Delta value is first added then subtracted from the red, green,",
-    "and blue of the target color.  Any pixels within the range also",
-    "have their matte value updated.",
+    "window to change its matte value.",
     "",
     "If the Magnify widget is mapped, it can be helpful in positioning",
     "your pointer within the image (refer to button 2).",
@@ -1359,9 +1395,10 @@ typedef enum
   ChopDismissCommand,
   HorizontalChopCommand,
   VerticalChopCommand,
-  ColorEditColorCommand,
   ColorEditMethodCommand,
-  ColorEditDeltaCommand,
+  ColorEditColorCommand,
+  ColorEditBorderCommand,
+  ColorEditFuzzCommand,
   ColorEditUndoCommand,
   ColorEditHelpCommand,
   ColorEditDismissCommand,
@@ -1383,7 +1420,8 @@ typedef enum
   DrawHelpCommand,
   DrawDismissCommand,
   MatteEditMethod,
-  MatteEditDeltaCommand,
+  MatteEditBorderCommand,
+  MatteEditFuzzCommand,
   MatteEditValueCommand,
   MatteEditUndoCommand,
   MatteEditHelpCommand,
@@ -1424,7 +1462,7 @@ typedef enum
 #define WavyWidth  16
 #define WavyHeight  16
 
-static unsigned char
+const unsigned char
   BricksBitmap[] =
   {
     0xff, 0xff, 0x0f, 0x03, 0x0c, 0x00, 0x03, 0x0c, 0x00, 0x03, 0x0c, 0x00,
@@ -1439,20 +1477,12 @@ static unsigned char
     0x11, 0x11, 0x22, 0x22, 0x44, 0x44, 0x88, 0x88, 0x11, 0x11, 0x22, 0x22,
     0x44, 0x44, 0x88, 0x88, 0x11, 0x11, 0x22, 0x22
   },
-  HighlightBitmap[] =
-  {
-    0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55
-  },
   ScalesBitmap[] =
   {
     0x08, 0x08, 0x08, 0x08, 0x14, 0x14, 0xe3, 0xe3, 0x80, 0x80, 0x80, 0x80,
     0x41, 0x41, 0x3e, 0x3e, 0x08, 0x08, 0x08, 0x08, 0x14, 0x14, 0xe3, 0xe3,
     0x80, 0x80, 0x80, 0x80, 0x41, 0x41, 0x3e, 0x3e
   },
-  ShadowBitmap[] =
-  {
-    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
-  },
   VerticalBitmap[] =
   {
     0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11,
@@ -1476,7 +1506,7 @@ static CommandType
 static Image
   *XMagickCommand(Display *,XResourceInfo *,XWindows *,const CommandType,
     Image **),
-  *XOpenImage(Display *,XResourceInfo *,XWindows *,const unsigned int),
+  *XOpenBlob(Display *,XResourceInfo *,XWindows *,const unsigned int),
   *XTileImage(Display *,XResourceInfo *,XWindows *,Image *,XEvent *),
   *XVisualDirectoryImage(Display *,XResourceInfo *,XWindows *);
 
@@ -1507,3 +1537,9 @@ static void
   XSetCropGeometry(Display *,XWindows *,RectangleInfo *,Image *),
   XScreenEvent(Display *,XWindows *,XEvent *),
   XTranslateImage(Display *,XWindows *,Image *,const KeySym);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
+#endif
