@@ -10,8 +10,7 @@
 %                           GGGG  EEEEE  M   M SSSSS                          %
 %                                                                             %
 %                                                                             %
-%                    Graphic Gems - Graphic Support Routines                  %
-%                                                                             %
+%                    Graphic Gems - Graphic Support Methods                   %
 %                                                                             %
 %                                                                             %
 %                               Software Design                               %
@@ -19,7 +18,7 @@
 %                                 August 1996                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1998 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -54,6 +53,7 @@
   Include declarations.
 */
 #include "magick.h"
+#include "defines.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -66,12 +66,12 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function Contrast enhances the intensity differences between the lighter
+%  Method Contrast enhances the intensity differences between the lighter
 %  and darker elements of the image.
 %
-%  The format of the ContrastImage routine is:
+%  The format of the Contrast method is:
 %
-%      Contrast(sign,red,green,blue)
+%      void Contrast(const int sign,Quantum *red,Quantum *green,Quantum *blue)
 %
 %  A description of each parameter follows:
 %
@@ -117,22 +117,22 @@ Export void Contrast(const int sign,Quantum *red,Quantum *green,Quantum *blue)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function GenerateNoise adds noise to a pixel.
+%  Method GenerateNoise adds noise to a pixel.
 %
-%  The format of the GenerateNoise routine is:
+%  The format of the GenerateNoise method is:
 %
-%      GenerateNoise(pixel,noise_type)
+%      Quantum GenerateNoise(const Quantum pixel,const NoiseType noise_type)
 %
 %  A description of each parameter follows:
 %
 %    o pixel: A structure of type Quantum.
 %
-%    o noise_type:  The type of noise: gaussian, multiplicative gaussian,
-%      impulse, laplacian, or poisson.
+%    o noise_type:  The type of noise: Gaussian, multiplicative Gaussian,
+%      impulse, laplacian, or Poisson.
 %
 %
 */
-Export Quantum GenerateNoise(Quantum pixel,NoiseType noise_type)
+Export Quantum GenerateNoise(const Quantum pixel,const NoiseType noise_type)
 {
 #define NoiseEpsilon  1.0e-5
 #define NoiseMask  0x7fff
@@ -244,12 +244,13 @@ Export Quantum GenerateNoise(Quantum pixel,NoiseType noise_type)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function HSLTransform converts a (hue, saturation, luminosity) to a
+%  Method HSLTransform converts a (hue, saturation, luminosity) to a
 %  (red, green, blue) triple.
 %
-%  The format of the HSLTransformImage routine is:
+%  The format of the HSLTransformImage method is:
 %
-%      HSLTransform(hue,saturation,luminosity,red,green,blue)
+%      void HSLTransform(const double hue,const double saturation,
+%        const double luminosity,Quantum *red,Quantum *green,Quantum *blue)
 %
 %  A description of each parameter follows:
 %
@@ -260,7 +261,7 @@ Export Quantum GenerateNoise(Quantum pixel,NoiseType noise_type)
 %
 %
 */
-Export void HSLTransform(double hue,const double saturation,
+Export void HSLTransform(const double hue,const double saturation,
   const double luminosity,Quantum *red,Quantum *green,Quantum *blue)
 {
   double
@@ -280,20 +281,17 @@ Export void HSLTransform(double hue,const double saturation,
   assert(blue != (Quantum *) NULL);
   v=(luminosity <= 0.5) ? (luminosity*(1.0+saturation)) :
     (luminosity+saturation-luminosity*saturation);
-  if (v <= 0)
+  if ((saturation == 0.0) || (hue == -1.0))
     {
-      *red=0;
-      *green=0;
-      *blue=0;
+      *red=(Quantum) floor((luminosity*(double) MaxRGB)+0.5);
+      *green=(Quantum) floor((luminosity*(double) MaxRGB)+0.5);
+      *blue=(Quantum) floor((luminosity*(double) MaxRGB)+0.5);
       return;
     }
-  if (hue == 1.0)
-    hue=0.0;
-  hue*=6.0;
-  y=luminosity+luminosity-v;
-  x=y+(v-y)*(hue-(int) hue);
-  z=v-(v-y)*(hue-(int) hue);
-  switch ((int) hue)
+  y=2*luminosity-v;
+  x=y+(v-y)*(6.0*hue-(int) (6.0*hue));
+  z=v-(v-y)*(6.0*hue-(int) (6.0*hue));
+  switch ((int) (6.0*hue))
   {
     default: r=v; g=x; b=y; break;
     case 0: r=v; g=x; b=y; break;
@@ -319,15 +317,16 @@ Export void HSLTransform(double hue,const double saturation,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function Hull implements the eight hull algorithm described in Applied
+%  Method Hull implements the eight hull algorithm described in Applied
 %  Optics, Vol. 24, No. 10, 15 May 1985, "Geometric filter for Speckle
 %  Reduction", by Thomas R Crimmins.  Each pixel in the image is replaced by
 %  one of its eight of its surrounding pixels using a polarity and negative
 %  hull function.
 %
-%  The format of the Hull routine is:
+%  The format of the Hull method is:
 %
-%      Hull(x_offset,y_offset,polarity,columns,rows,f,g)
+%      void Hull(const int x_offset,const int y_offset,const int polarity,
+%        const unsigned int columns,const unsigned int rows,Quantum *f,Quantum *g)
 %
 %  A description of each parameter follows:
 %
@@ -342,8 +341,8 @@ Export void HSLTransform(double hue,const double saturation,
 %
 %
 */
-Export void Hull(int x_offset,int y_offset,int polarity,unsigned int columns,
-  unsigned int rows,Quantum *f,Quantum *g)
+Export void Hull(const int x_offset,const int y_offset,const int polarity,
+  const unsigned int columns,const unsigned int rows,Quantum *f,Quantum *g)
 {
   int
     y;
@@ -365,13 +364,13 @@ Export void Hull(int x_offset,int y_offset,int polarity,unsigned int columns,
   p=f+(columns+2);
   q=g+(columns+2);
   r=p+(y_offset*((int) columns+2)+x_offset);
-  for (y=0; y < rows; y++)
+  for (y=0; y < (int) rows; y++)
   {
     p++;
     q++;
     r++;
     if (polarity > 0)
-      for (x=0; x < columns; x++)
+      for (x=0; x < (int) columns; x++)
       {
         v=(*p);
         if (*r > v)
@@ -382,7 +381,7 @@ Export void Hull(int x_offset,int y_offset,int polarity,unsigned int columns,
         r++;
       }
     else
-      for (x=0; x < columns; x++)
+      for (x=0; x < (int) columns; x++)
       {
         v=(*p);
         if (v > (Quantum) (*r+1))
@@ -400,14 +399,14 @@ Export void Hull(int x_offset,int y_offset,int polarity,unsigned int columns,
   q=g+(columns+2);
   r=q+(y_offset*((int) columns+2)+x_offset);
   s=q-(y_offset*((int) columns+2)+x_offset);
-  for (y=0; y < rows; y++)
+  for (y=0; y < (int) rows; y++)
   {
     p++;
     q++;
     r++;
     s++;
     if (polarity > 0)
-      for (x=0; x < columns; x++)
+      for (x=0; x < (int) columns; x++)
       {
         v=(*q);
         if (((Quantum) (*s+1) > v) && (*r > v))
@@ -419,7 +418,7 @@ Export void Hull(int x_offset,int y_offset,int polarity,unsigned int columns,
         s++;
       }
     else
-      for (x=0; x < columns; x++)
+      for (x=0; x < (int) columns; x++)
       {
         v=(*q);
         if (((Quantum) (*s+1) < v) && (*r < v))
@@ -442,437 +441,41 @@ Export void Hull(int x_offset,int y_offset,int polarity,unsigned int columns,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   I n s i d e P r i m i t i v e                                             %
+%   I n t e r p o l a t e C o l o r                                           %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function InsidePrimitive returns True if the (x,y) position is inside the
-%  primitive as defined in primitive_info.  Otherwise False is returned.
-%
-%  The format of the InsidePrimitive routine is:
-%
-%      status=InsidePrimitive(primitive_info,annotate_info,x,y,image)
-%
-%  A description of each parameter follows:
-%
-%    o status:  Function InsidePrimitive returns True if the (x,y) position
-%      is inside the primitive as defined in primitive_info.  Otherwise False
-%      is returned.
-%
-%    o primitive_info: Specifies a pointer to a PrimitiveInfo structure.
-%
-%    o annotate_info: Specifies a pointer to a AnnotateInfo structure.
-%
-%    o x,y: Integers representing the (x,y) location in the image.
-%
-%    o image: The address of a structure of type Image.
-%
-%
-*/
-Export unsigned int InsidePrimitive(PrimitiveInfo *primitive_info,
-  AnnotateInfo *annotate_info,int x,int y,Image *image)
-{
-  float
-    mid,
-    slope,
-    trix,
-    triy;
-
-  register int
-    i;
-
-  register PrimitiveInfo
-    *p,
-    *q;
-
-  register unsigned int
-    inside;
-
-  assert(primitive_info != (PrimitiveInfo *) NULL);
-  assert(annotate_info != (AnnotateInfo *) NULL);
-  assert(image != (Image *) NULL);
-  inside=False;
-  mid=annotate_info->linewidth/2.0;
-  p=primitive_info;
-  while (p->primitive != UndefinedPrimitive)
-  {
-    q=p+p->coordinates-1;
-    switch (p->primitive)
-    {
-      case PointPrimitive:
-      default:
-      {
-        for ( ; (p < q) && !inside; p++)
-          inside=(x == p->x) && (y == p->y);
-        break;
-      }
-      case LinePrimitive:
-      {
-        for ( ; (p < q) && !inside; p++)
-        {
-          if (p->x == (p+1)->x)
-            inside=(x >= (p->x-mid)) && (x < (p->x+mid)) &&
-              (y >= Min(p->y,(p+1)->y)) && (y <= Max(p->y,(p+1)->y));
-          else
-            if (p->y == (p+1)->y)
-              inside=(x >= Min(p->x,(p+1)->x)) && (x <= Max(p->x,(p+1)->x)) &&
-                (y >= (p->y-mid)) && (y < (p->y+mid));
-            else
-              {
-                slope=(float) (p->y-(p+1)->y)/(p->x-(p+1)->x);
-                trix=(slope*p->x+x/slope+y-p->y)/(slope+1.0/slope);
-                triy=slope*(trix-p->x)+p->y;
-                inside=(((x-trix)*(x-trix)+(y-triy)*(y-triy)) <= (mid*mid)) &&
-                  (trix >= Min(p->x,(p+1)->x)) && (trix <= Max(p->x,(p+1)->x));
-              }
-          p++;
-        }
-        break;
-      }
-      case RectanglePrimitive:
-      {
-        for ( ; (p < q) && !inside; p++)
-        {
-          inside=(x >= Min(p->x-mid,(p+1)->x+mid)) &&
-            (x < Max(p->x-mid,(p+1)->x+mid)) &&
-            (y >= Min(p->y-mid,(p+1)->y+mid)) &&
-            (y < Max(p->y-mid,(p+1)->y+mid));
-          inside&=!((x >= Min(p->x+mid,(p+1)->x-mid)) &&
-            (x < Max(p->x+mid,(p+1)->x-mid)) &&
-            (y >= Min(p->y+mid,(p+1)->y-mid)) &&
-            (y < Max(p->y+mid,(p+1)->y-mid)));
-          p++;
-        }
-        break;
-      }
-      case FillRectanglePrimitive:
-      {
-        for ( ; (p < q) && !inside; p++)
-        {
-          inside=(x >= Min(p->x,(p+1)->x)) && (x <= Max(p->x,(p+1)->x)) &&
-            (y >= Min(p->y,(p+1)->y)) && (y <= Max(p->y,(p+1)->y));
-          p++;
-        }
-        break;
-      }
-      case EllipsePrimitive:
-      {
-        for ( ; (p < q) && !inside; p++)
-        {
-          inside=(((p->y-y)*(p->y-y))+((p->x-x)*(p->x-x))) <=
-            (((p->y-(p+1)->y-mid)*(p->y-(p+1)->y-mid))+
-            ((p->x-(p+1)->x-mid)*(p->x-(p+1)->x-mid)));
-          inside&=!((((p->y-y)*(p->y-y))+((p->x-x)*(p->x-x))) <=
-            (((p->y-(p+1)->y+mid)*(p->y-(p+1)->y+mid))+
-            ((p->x-(p+1)->x+mid)*(p->x-(p+1)->x+mid))));
-          p++;
-        }
-        break;
-      }
-      case FillEllipsePrimitive:
-      {
-        for ( ; (p < q) && !inside; p++)
-        {
-          inside=((p->y-y)*(p->y-y))+((p->x-x)*(p->x-x)) <=
-            ((p->y-(p+1)->y)*(p->y-(p+1)->y))+((p->x-(p+1)->x)*(p->x-(p+1)->x));
-          p++;
-        }
-        break;
-      }
-      case PolygonPrimitive:
-      {
-        for ( ; (p < q) && !inside; p++)
-        {
-          if (p->x == (p+1)->x)
-            {
-              inside|=(x >= (p->x-mid)) && (x < (p->x+mid)) &&
-                (y >= Min(p->y,(p+1)->y)) && (y <= Max(p->y,(p+1)->y));
-              continue;
-            }
-          if (p->y == (p+1)->y)
-            {
-              inside|=(x >= Min(p->x,(p+1)->x)) && (x <= Max(p->x,(p+1)->x)) &&
-                (y >= (p->y-mid)) && (y < (p->y+mid));
-              continue;
-            }
-          slope=(float) (p->y-(p+1)->y)/(p->x-(p+1)->x);
-          trix=(slope*p->x+x/slope+y-p->y)/(slope+1.0/slope);
-          triy=slope*(trix-p->x)+p->y;
-          inside|=(((x-trix)*(x-trix)+(y-triy)*(y-triy)) <= (mid*mid)) &&
-            (trix >= Min(p->x,(p+1)->x)) && (trix <= Max(p->x,(p+1)->x));
-        }
-        while (p <= q)
-          p++;
-        if (inside)
-          break;
-        p--;
-        q=primitive_info;
-        if (p->x == q->x)
-          inside=(x >= (p->x-mid)) && (x < (p->x+mid)) &&
-            (y >= Min(p->y,q->y)) && (y <= Max(p->y,q->y));
-        else
-          if (p->y == q->y)
-            inside=(x >= Min(p->x,q->x)) && (x <= Max(p->x,q->x)) &&
-              (y >= (p->y-mid)) && (y < (p->y+mid));
-          else
-            {
-              slope=(float) (p->y-q->y)/(p->x-q->x);
-              trix=(slope*p->x+x/slope+y-p->y)/(slope+1.0/slope);
-              triy=slope*(trix-p->x)+p->y;
-              inside=(((x-trix)*(x-trix)+(y-triy)*(y-triy)) <= (mid*mid)) &&
-                (trix >= Min(p->x,q->x)) && (trix <= Max(p->x,q->x));
-            }
-        p++;
-        break;
-      }
-      case FillPolygonPrimitive:
-      {
-        int
-          crossing,
-          crossings;
-
-        crossings=0;
-        if ((q->y >= y) != (p->y >= y))
-          {
-            crossing=q->x >= x;
-            if (crossing != (p->x >= x))
-              crossings+=(q->x-(q->y-y)*(p->x-q->x)/(p->y-q->y)) >= x;
-            else
-              if (crossing)
-                crossings++;
-          }
-        for (p++; p <= q; p++)
-        {
-          if ((p-1)->y >= y)
-            {
-              while ((p <= q) && (p->y >= y))
-                p++;
-              if (p > q)
-                break;
-              crossing=(p-1)->x >= x;
-              if (crossing != (p->x >= x))
-                crossings+=
-                  ((p-1)->x-((p-1)->y-y)*(p->x-(p-1)->x)/(p->y-(p-1)->y)) >= x;
-              else
-                if (crossing)
-                  crossings++;
-              continue;
-            }
-         while ((p <= q) && (p->y < y))
-           p++;
-         if (p > q)
-           break;
-         crossing=(p-1)->x >= x;
-         if (crossing != (p->x >= x))
-           crossings+=
-             ((p-1)->x-((p-1)->y-y)*(p->x-(p-1)->x)/(p->y-(p-1)->y)) >= x;
-         else
-           if (crossing)
-             crossings++;
-        }
-        inside=crossings & 0x01;
-        break;
-      }
-      case ColorPrimitive:
-      {
-        for ( ; (p <= q) && !inside; p++)
-          switch (p->method)
-          {
-            case PointMethod:
-            default:
-            {
-              if ((p->x != x) || (p->y != y))
-                break;
-              inside=True;
-              break;
-            }
-            case ReplaceMethod:
-            {
-              RunlengthPacket
-                color;
-
-              static RunlengthPacket
-                target;
-
-              if ((x == 0) && (y == 0))
-                target=image->pixels[p->y*image->columns+p->x];
-              color=image->pixels[y*image->columns+x];
-              inside=ColorMatch(color,target,0);
-              break;
-            }
-            case FloodfillMethod:
-            {
-              ColorPacket
-                color;
-
-              XColor
-                pen_color;
-
-              if ((p->x != x) || (p->y != y))
-                break;
-              (void) XQueryColorDatabase(annotate_info->image_info->pen,
-                &pen_color);
-              color.red=XDownScale(pen_color.red);
-              color.green=XDownScale(pen_color.green);
-              color.blue=XDownScale(pen_color.blue);
-              ColorFloodfillImage(image,x,y,&color,0);
-              break;
-            }
-            case ResetMethod:
-            {
-              inside=True;
-              break;
-            }
-          }
-        break;
-      }
-      case MattePrimitive:
-      {
-        if (!image->matte)
-          {
-            /*
-              Initialize matte image.
-            */
-            image->matte=True;
-            for (i=0; i < image->packets; i++)
-              image->pixels[i].index=Opaque;
-          }
-        for ( ; p <= q; p++)
-          switch (p->method)
-          {
-            case PointMethod:
-            default:
-            {
-              if ((p->x != x) || (p->y != y))
-                break;
-              image->pixels[y*image->columns+x].index=Transparent;
-              break;
-            }
-            case ReplaceMethod:
-            {
-              RunlengthPacket
-                color;
-
-              static RunlengthPacket
-                target;
-
-              if ((x == 0) && (y == 0))
-                target=image->pixels[p->y*image->columns+p->x];
-              color=image->pixels[y*image->columns+x];
-              if (ColorMatch(color,target,0))
-                image->pixels[y*image->columns+x].index=Transparent;
-              break;
-            }
-            case FloodfillMethod:
-            {
-              if ((p->x != x) || (p->y != y))
-                break;
-              MatteFloodfillImage(image,x,y,Transparent,0);
-              break;
-            }
-            case ResetMethod:
-            {
-              image->pixels[y*image->columns+x].index=Transparent;
-              break;
-            }
-          }
-        break;
-      }
-      case TextPrimitive:
-      case ImagePrimitive:
-      {
-        register char
-          *r;
-
-        for ( ; p <= q; p++)
-        {
-          if ((p->x != x) || (p->y != y))
-            continue;
-          if (p->text == (char *) NULL)
-            continue;
-          r=p->text;
-          if (*r == '"')
-            {
-              p->text++;
-              for (r++; (*r != '"') && (*r != '\0'); r++);
-            }
-          else
-            if (*r == '\'')
-              {
-                p->text++;
-                for (r++; (*r != '\'') && (*r != '\0'); r++);
-              }
-            else
-              for (r++; (*r != ' ') && (*r != '\0'); r++);
-          (void) strncpy(annotate_info->text,p->text,r-p->text);
-          annotate_info->text[r-p->text]='\0';
-          if (p->primitive == TextPrimitive)
-            {
-              (void) sprintf(annotate_info->geometry,"%+d%+d",p->x,p->y);
-              AnnotateImage(image,annotate_info);
-            }
-          else
-            {
-              Image
-                *composite_image;
-
-              ImageInfo
-                composite_info;
-
-              GetImageInfo(&composite_info);
-              (void) strcpy(composite_info.filename,annotate_info->text);
-              composite_image=ReadImage(&composite_info);
-              if (composite_image != (Image *) NULL)
-                {
-                  CompositeImage(image,ReplaceCompositeOp,composite_image,
-                    p->x,p->y);
-                  DestroyImage(composite_image);
-                }
-              DestroyImageInfo(&composite_info);
-            }
-        }
-        break;
-      }
-    }
-    if (inside)
-      return(True);
-  }
-  return(inside);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   I n t e r p o l a t e                                                     %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  Function Interpolate applies bi-linear interpolation between a pixel and
+%  Method InterpolateColor applies bi-linear interpolation between a pixel and
 %  it's neighbors.
 %
-%  The format of the Interpolate routine is:
+%  The format of the InterpolateColor method is:
 %
-%      Interpolate(image,pixel,x,y)
+%      ColorPacket InterpolateColor(Image *image,const double x_offset,
+%        const double y_offset)
 %
 %  A description of each parameter follows:
 %
 %    o image: The address of a structure of type Image.
 %
-%    o pixel: A pointer to a RunlengthPacket representing the current pixel.
-%
-%    o x,y: A double representing the current (x,y) position of the pixel.
+%    o x_offset,y_offset: A double representing the current (x,y) position of
+%      the pixel.
 %
 %
 */
-Export RunlengthPacket Interpolate(Image *image,RunlengthPacket *pixel,double x,
-  double y)
+Export ColorPacket InterpolateColor(Image *image,const double x_offset,
+  const double y_offset)
 {
+  ColorPacket
+    interpolated_pixel;
+
+  double
+    alpha,
+    beta,
+    x,
+    y;
+
   register RunlengthPacket
     *p,
     *q,
@@ -880,35 +483,64 @@ Export RunlengthPacket Interpolate(Image *image,RunlengthPacket *pixel,double x,
     *s;
 
   RunlengthPacket
-    interpolated_pixel;
+    background_pixel;
 
   assert(image != (Image *) NULL);
-  assert(pixel != (RunlengthPacket *) NULL);
-  if ((x < 0) || (x >= image->columns) || (y < 0) || (y >= image->rows))
-    return(*pixel);
-  if (!UncondenseImage(image))
-    return(*pixel);
-  p=image->pixels+(int) y*image->columns+(int) x;
-  q=p+1;
-  if (q > (image->pixels+image->packets-1))
-    q=p;
-  r=p+image->columns;
-  if (r > (image->pixels+image->packets-1))
-    r=p;
-  s=q+image->columns;
-  if (s > (image->pixels+image->packets-1))
-    s=q;
-  x=fmod(x,1.0);
-  y=fmod(y,1.0);
+  if (image->packets != (image->columns*image->rows))
+    if (!UncondenseImage(image))
+      return(image->background_color);
+  if ((x_offset < -1) || (x_offset >= image->columns) ||
+      (y_offset < -1) || (y_offset >= image->rows))
+    return(image->background_color);
+  background_pixel.red=image->background_color.red;
+  background_pixel.green=image->background_color.green;
+  background_pixel.blue=image->background_color.blue;
+  background_pixel.index=image->background_color.index;
+  x=x_offset;
+  y=y_offset;
+  if ((x >= 0) && (y >= 0))
+    {
+      p=image->pixels+((int) y)*image->columns+(int) x;
+      q=p+1;
+      if ((x+1) >= image->columns)
+        q=(&background_pixel);
+      r=p+image->columns;
+      if ((y+1) >= image->rows)
+        r=(&background_pixel);
+      s=p+image->columns+1;
+      if (((x+1) >= image->columns) || ((y+1) >= image->rows))
+        s=(&background_pixel);
+    }
+  else
+    {
+      p=(&background_pixel);
+      q=(&background_pixel);
+      r=image->pixels+(int) x;
+      s=r+1;
+      if ((x >= -1) && (x < 0))
+        {
+          q=image->pixels+(int) y*image->columns;
+          r=(&background_pixel);
+          s=q+(int) image->columns;
+          if ((y >= -1) && (y < 0))
+            {
+              q=(&background_pixel);
+              s=image->pixels;
+            }
+        }
+    }
+  x-=floor(x);
+  y-=floor(y);
+  alpha=1.0-x;
+  beta=1.0-y;
   interpolated_pixel.red=(Quantum)
-    ((1.0-y)*((1.0-x)*p->red+x*q->red)+y*((1.0-x)*r->red+x*s->red));
+    (beta*(alpha*p->red+x*q->red)+y*(alpha*r->red+x*s->red));
   interpolated_pixel.green=(Quantum)
-    ((1.0-y)*((1.0-x)*p->green+x*q->green)+y*((1.0-x)*r->green+x*s->green));
+    (beta*(alpha*p->green+x*q->green)+y*(alpha*r->green+x*s->green));
   interpolated_pixel.blue=(Quantum)
-    ((1.0-y)*((1.0-x)*p->blue+x*q->blue)+y*((1.0-x)*r->blue+x*s->blue));
+    (beta*(alpha*p->blue+x*q->blue)+y*(alpha*r->blue+x*s->blue));
   interpolated_pixel.index=(unsigned short)
-    ((1.0-y)*((1.0-x)*p->index+x*q->index)+y*((1.0-x)*r->index+x*s->index));
-  interpolated_pixel.length=p->length;
+    (beta*(alpha*p->index+x*q->index)+y*(alpha*r->index+x*s->index));
   return(interpolated_pixel);
 }
 
@@ -923,17 +555,18 @@ Export RunlengthPacket Interpolate(Image *image,RunlengthPacket *pixel,double x,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function Modulate modulates the hue, saturation, and brightness of an
+%  Method Modulate modulates the hue, saturation, and brightness of an
 %  image.
 %
-%  The format of the ModulateImage routine is:
+%  The format of the ModulateImage method is:
 %
-%      Modulate(percent_hue,percent_saturation,percent_luminosity,red,green,
+%      void Modulate(double percent_hue,double percent_saturation,
+%        double percent_brightness,Quantum *red,Quantum *green,Quantum *blue)
 %        blue)
 %
 %  A description of each parameter follows:
 %
-%    o percent_hue, percent_saturation, percent_luminosity: A double value
+%    o percent_hue, percent_saturation, percent_brightness: A double value
 %      representing the percent change in a component of the HSL color space.
 %
 %    o red, green, blue: A pointer to a pixel component of type Quantum.
@@ -990,12 +623,13 @@ Export void Modulate(double percent_hue,double percent_saturation,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function TransformHSL converts a (red, green, blue) to a (hue, saturation,
+%  Method TransformHSL converts a (red, green, blue) to a (hue, saturation,
 %  luminosity) triple.
 %
-%  The format of the TransformHSL routine is:
+%  The format of the TransformHSL method is:
 %
-%      TransformHSL(red,green,blue,hue,saturation,luminosity)
+%      void TransformHSL(const Quantum red,const Quantum green,
+%        const Quantum blue,double *hue,double *saturation,double *luminosity)
 %
 %  A description of each parameter follows:
 %
@@ -1012,6 +646,7 @@ Export void TransformHSL(const Quantum red,const Quantum green,
 {
   double
     b,
+    delta,
     g,
     max,
     min,
@@ -1023,27 +658,25 @@ Export void TransformHSL(const Quantum red,const Quantum green,
   assert(hue != (double *) NULL);
   assert(saturation != (double *) NULL);
   assert(luminosity != (double *) NULL);
-  *hue=1.0;
-  *saturation=0.0;
   r=(double) red/(double) MaxRGB;
   g=(double) green/(double) MaxRGB;
   b=(double) blue/(double) MaxRGB;
   max=Max(r,Max(g,b));
   min=Min(r,Min(g,b));
+  *hue=(-1.0);
+  *saturation=0.0;
   *luminosity=(min+max)/2.0;
-  if (*luminosity <= 0.0)
+  delta=max-min;
+  if (delta == 0.0)
     return;
-  *saturation=max-min;
-  if (*saturation <= 0.0)
-    return;
-  *saturation/=(*luminosity <= 0.5) ? (min+max) : (2.0-max-min) ;
+  *saturation=delta/((*luminosity <= 0.5) ? (min+max) : (2.0-max-min));
   if (r == max)
-    *hue=(g == min ? 5.0+(max-b)/(max-min) : 1.0-(max-g)/(max-min));
+    *hue=(g == min ? 5.0+(max-b)/delta : 1.0-(max-g)/delta);
   else
     if (g == max)
-      *hue=(b == min ? 1.0+(max-r)/(max-min) : 3.0-(max-b)/(max-min));
+      *hue=(b == min ? 1.0+(max-r)/delta : 3.0-(max-b)/delta);
     else
-      *hue=(r == min ? 3.0+(max-g)/(max-min) : 5.0-(max-r)/(max-min));
+      *hue=(r == min ? 3.0+(max-g)/delta : 5.0-(max-r)/delta);
   *hue/=6.0;
 }
 
@@ -1058,15 +691,16 @@ Export void TransformHSL(const Quantum red,const Quantum green,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function Upsample
+%  Method Upsample doubles the size of the image.
 %
-%  The format of the UpSample routine is:
+%  The format of the UpSample method is:
 %
-%      Upsample(width,height,scaled_width,pixels)
+%      void Upsample(const unsigned int width,const unsigned int height,
+%        const unsigned int scaled_width,unsigned char *pixels)
 %
 %  A description of each parameter follows:
 %
-%    o width,height:  Unsigned values representinf the width and height of
+%    o width,height:  Unsigned values representing the width and height of
 %      the image pixel array.
 %
 %    o scaled_width:  Specifies the final width of the upsampled pixel array.
@@ -1076,8 +710,8 @@ Export void TransformHSL(const Quantum red,const Quantum green,
 %
 %
 */
-Export void Upsample(unsigned int width,unsigned int height,
-  unsigned int scaled_width,unsigned char *pixels)
+Export void Upsample(const unsigned int width,const unsigned int height,
+  const unsigned int scaled_width,unsigned char *pixels)
 {
   register int
     x,
@@ -1092,13 +726,13 @@ Export void Upsample(unsigned int width,unsigned int height,
     Create a new image that is a integral size greater than an existing one.
   */
   assert(pixels != (unsigned char *) NULL);
-  for (y=0; y < height; y++)
+  for (y=0; y < (int) height; y++)
   {
     p=pixels+(height-1-y)*scaled_width+(width-1);
     q=pixels+((height-1-y) << 1)*scaled_width+((width-1) << 1);
     *q=(*p);
     *(q+1)=(*(p));
-    for (x=1; x < width; x++)
+    for (x=1; x < (int) width; x++)
     {
       p--;
       q-=2;
@@ -1106,12 +740,12 @@ Export void Upsample(unsigned int width,unsigned int height,
       *(q+1)=(((int) *p)+((int) *(p+1))+1) >> 1;
     }
   }
-  for (y=0; y < (height-1); y++)
+  for (y=0; y < (int) (height-1); y++)
   {
     p=pixels+(y << 1)*scaled_width;
     q=p+scaled_width;
     r=q+scaled_width;
-    for (x=0; x < (width-1); x++)
+    for (x=0; x < (int) (width-1); x++)
     {
       *q=(((int) *p)+((int) *r)+1) >> 1;
       *(q+1)=(((int) *p)+((int) *(p+2))+((int) *r)+((int) *(r+2))+2) >> 2;
@@ -1124,7 +758,7 @@ Export void Upsample(unsigned int width,unsigned int height,
   }
   p=pixels+(2*height-2)*scaled_width;
   q=pixels+(2*height-1)*scaled_width;
-  for (x=0; x < width; x++)
+  for (x=0; x < (int) width; x++)
   {
     *q++=(*p++);
     *q++=(*p++);
