@@ -3,14 +3,14 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%                                                                             %
 %                                 N   N  TTTTT                                %
 %                                 NN  N    T                                  %
 %                                 N N N    T                                  %
 %                                 N  NN    T                                  %
 %                                 N   N    T                                  %
 %                                                                             %
-%                 Windows NT Utility Routines for ImageMagick.                %
+%                                                                             %
+%                  Windows NT Utility Methods for ImageMagick                 %
 %                                                                             %
 %                                                                             %
 %                               Software Design                               %
@@ -18,7 +18,7 @@
 %                                December 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1998 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -52,9 +52,8 @@
 /*
   Include declarations.
 */
-#include <stdio.h>
-#include <stdlib.h>
-#include <assert.h>
+#include "magick.h"
+#include "defines.h"
 #include "nt.h"
 
 /*
@@ -62,7 +61,7 @@
 */
 #if !defined(_VISUALC_)
 extern "C" BOOL WINAPI
-  DllMain(HINSTANCE hInst,DWORD wDataSeg,LPVOID lpvReserved);
+  DllMain(HINSTANCE instance,DWORD data_segment,LPVOID reserved);
 #endif
 
 extern char
@@ -79,12 +78,12 @@ extern char
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function closedir closes the named directory stream and frees the DIR
+%  Method closedir closes the named directory stream and frees the DIR
 %  structure.
 %
-%  The format of the closedir routine is:
+%  The format of the closedir method is:
 %
-%      closedir(entry)
+%      void closedir(DIR *entry)
 %
 %  A description of each parameter follows:
 %
@@ -92,11 +91,11 @@ extern char
 %
 %
 */
-void closedir(DIR *entry)
+Export void closedir(DIR *entry)
 {
   assert(entry != (DIR *) NULL);
   FindClose(entry->hSearch);
-  free((char *) entry);
+  FreeMemory((char *) entry);
 }
 
 /*
@@ -110,11 +109,11 @@ void closedir(DIR *entry)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function Exit calls TerminateProcess for Win95.
+%  Method Exit calls TerminateProcess for Win95.
 %
-%  The format of the Exit routine is:
+%  The format of the Exit method is:
 %
-%      Exit(status)
+%      int Exit(int status)
 %
 %  A description of each parameter follows:
 %
@@ -123,9 +122,9 @@ void closedir(DIR *entry)
 %
 %
 */
-__declspec(dllexport) int Exit(int status)
+Export int Exit(int status)
 {
-  if (IsWin95())
+  if (IsWindows95())
     TerminateProcess(GetCurrentProcess(),(unsigned int) status);
   exit(status);
   return(0);
@@ -142,27 +141,27 @@ __declspec(dllexport) int Exit(int status)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function ImageFormatConflict returns true if the image format conflicts with
+%  Method ImageFormatConflict returns true if the image format conflicts with
 %  a logical drive (.e.g. X:).
 %
-%  The format of the ImageFormatConflict routine is:
+%  The format of the ImageFormatConflict method is:
 %
-%      status=ImageFormatConflict(magick)
+%      int ImageFormatConflict(const char *magick)
 %
 %  A description of each parameter follows:
 %
-%    o status: Function ImageFormatConflict returns true if the image format
-%              conflicts with a logical drive.
+%    o status: Method ImageFormatConflict returns true if the image format
+%      conflicts with a logical drive.
 %
 %    o magick: Specifies the image format.
 %
 %
 */
-int ImageFormatConflict(char *magick)
+Export int ImageFormatConflict(const char *magick)
 {
   assert(magick != (char *) NULL);
   if (strlen(magick) > 1)
-    return(0);
+    return(False);
   return((GetLogicalDrives()) & (1 << ((toupper((int) (*magick)))-'A')));
 }
 
@@ -171,17 +170,17 @@ int ImageFormatConflict(char *magick)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   I s W i n 9 5                                                             %
+%   I s W i n d o w s 9 5                                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function IsWin95 returns true if the system is Windows 95.
+%  Method IsWindows95 returns true if the system is Windows 95.
 %
-%  The format of the IsWin95 routine is:
+%  The format of the IsWindows95 method is:
 %
-%      IsWin95()
+%      int IsWindows95()
 %
 %  A description of each parameter follows:
 %
@@ -190,7 +189,7 @@ int ImageFormatConflict(char *magick)
 %
 %
 */
-__declspec(dllexport) int IsWin95()
+Export int IsWindows95()
 {
   OSVERSIONINFO
     version_info;
@@ -213,15 +212,18 @@ __declspec(dllexport) int IsWin95()
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function NTErrorHandler displays an error message and then terminates
+%  Method NTErrorHandler displays an error message and then terminates
 %  the program.
 %
-%  The format of the NTErrorHandler routine is:
+%  The format of the NTErrorHandler method is:
 %
-%      NTErrorHandler(message,qualifier)
+%      void NTErrorHandler(const unsigned int error,const char *message,
+%        const char *qualifier)
 %
 %  A description of each parameter follows:
 %
+%    o error: Specifies the numeric error category.
+%
 %    o message: Specifies the message to display before terminating the
 %      program.
 %
@@ -229,19 +231,28 @@ __declspec(dllexport) int IsWin95()
 %
 %
 */
-void NTErrorHandler(const char *message,const char *qualifier)
+Export void NTErrorHandler(const unsigned int error,const char *message,
+  const char *qualifier)
 {
   char
-    buffer[2048];
+    buffer[3*MaxTextExtent];
 
   if (message == (char *) NULL)
     Exit(0);
-  (void) sprintf(buffer,"%s: %s",SetClientName((char *) NULL),message);
-  if (qualifier != (char *) NULL)
-    (void) sprintf(buffer,"%s (%s)",buffer,qualifier);
-  if (errno)
-    (void) sprintf(buffer,"%s [%s]",buffer,strerror(errno));
-  (void) sprintf(buffer,"%s.\n",buffer);
+  if ((qualifier != (char *) NULL) && errno)
+    FormatString(buffer,"%.1024s: %.1024s (%.1024s) [%.1024s].\n",
+      SetClientName((char *) NULL),message,qualifier,strerror(errno));
+  else
+    if (qualifier != (char *) NULL)
+      FormatString(buffer,"%.1024s: %.1024s (%.1024s).\n",
+        SetClientName((char *) NULL),message,qualifier);
+    else
+      if (errno)
+        FormatString(buffer,"%.1024s: %.1024s [%.1024s].\n",
+          SetClientName((char *) NULL),message,strerror(errno));
+      else
+        FormatString(buffer,"%.1024s: %.1024s.\n",SetClientName((char *) NULL),
+          message);
   (void) MessageBox(NULL,buffer,"ImageMagick Error",MB_OK | MB_TASKMODAL |
     MB_SETFOREGROUND | MB_ICONEXCLAMATION);
   Exit(0);
@@ -258,12 +269,12 @@ void NTErrorHandler(const char *message,const char *qualifier)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%   Function NTSystemCommand executes the specified command and waits until it
+%   Method NTSystemCommand executes the specified command and waits until it
 %   terminates.  The returned value is the exit status of the command.
 %
-%  The format of the NTSystemCommand routine is:
+%  The format of the NTSystemCommand method is:
 %
-%      NTSystemCommand(command)
+%      int NTSystemCommand(const char *command)
 %
 %  A description of each parameter follows:
 %
@@ -271,7 +282,7 @@ void NTErrorHandler(const char *message,const char *qualifier)
 %
 %
 */
-__declspec(dllexport) int NTSystemCommand(char *command)
+Export int NTSystemCommand(const char *command)
 {
   char
     local_command[2048];
@@ -334,12 +345,12 @@ __declspec(dllexport) int NTSystemCommand(char *command)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%   Function NTTemporaryFilename creates a name for a temporary file.  It
+%   Method NTTemporaryFilename creates a name for a temporary file.  It
 %   returns zero if an error occurs.
 %
-%  The format of the TemporaryFilename routine is:
+%  The format of the TemporaryFilename method is:
 %
-%      NTTemporaryFilename(filename)
+%      int NTTemporaryFilename(char *filename)
 %
 %  A description of each parameter follows:
 %
@@ -347,7 +358,7 @@ __declspec(dllexport) int NTSystemCommand(char *command)
 %
 %
 */
-int NTTemporaryFilename(char *filename)
+Export int NTTemporaryFilename(char *filename)
 {
   char
     path[1024];
@@ -359,6 +370,10 @@ int NTTemporaryFilename(char *filename)
   status=GetTempPath(sizeof(path),path);
   if (status != 0)
     status=GetTempFileName(path,"magick",0,filename);
+  (void) remove(filename);
+  if (strlen(filename) > 4)
+    if (strcmp(filename+strlen(filename)-4,".tmp") == 0)
+      filename[strlen(filename)-4]='\0';
   return(status);
 }
 
@@ -373,14 +388,17 @@ int NTTemporaryFilename(char *filename)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function NTWarningHandler displays a warning message.
+%  Method NTWarningHandler displays a warning message.
 %
-%  The format of the NTWarningHandler routine is:
+%  The format of the NTWarningHandler method is:
 %
-+      NTWarningHandler(message,qualifier)
+%      void NTWarningHandler(const unsigned int warning,const char *message,
+%        const char *qualifier)
 %
 %  A description of each parameter follows:
 %
+%    o warning: Specifies the numeric warning category.
+%
 %    o message: Specifies the message to display before terminating the
 %      program.
 %
@@ -388,17 +406,20 @@ int NTTemporaryFilename(char *filename)
 %
 %
 */
-void NTWarningHandler(const char *message,const char *qualifier)
+Export void NTWarningHandler(const unsigned int warning,const char *message,
+  const char *qualifier)
 {
   char
     buffer[2048];
 
   if (message == (char *) NULL)
     return;
-  (void) sprintf(buffer,"%s: %s",SetClientName((char *) NULL),message);
-  if (qualifier != (char *) NULL)
-    (void) sprintf(buffer,"%s (%s)",buffer,qualifier);
-  (void) sprintf(buffer,"%s.\n",buffer);
+  if (qualifier == (char *) NULL)
+    FormatString(buffer,"%.1024s: %.1024s.\n",
+      SetClientName((char *) NULL),message);
+  else
+    FormatString(buffer,"%.1024s: %.1024s (%.1024s).\n",
+      SetClientName((char *) NULL),message,qualifier);
   (void) MessageBox(NULL,buffer,"ImageMagick Warning",MB_OK | MB_TASKMODAL |
     MB_SETFOREGROUND | MB_ICONINFORMATION);
 }
@@ -414,12 +435,12 @@ void NTWarningHandler(const char *message,const char *qualifier)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function opendir opens the directory named by filename and associates
+%  Method opendir opens the directory named by filename and associates
 %  a directory stream with it.
 %
-%  The format of the opendir routine is:
+%  The format of the opendir method is:
 %
-%      opendir(entry)
+%      DIR *opendir(char *path)
 %
 %  A description of each parameter follows:
 %
@@ -427,7 +448,7 @@ void NTWarningHandler(const char *message,const char *qualifier)
 %
 %
 */
-DIR *opendir(char *path)
+Export DIR *opendir(char *path)
 {
   char
     file_specification[2048];
@@ -438,7 +459,7 @@ DIR *opendir(char *path)
   assert(path != (char *) NULL);
   (void) strcpy(file_specification,path);
   (void) strcat(file_specification,"/*.*");
-  entry=(DIR *) malloc(sizeof(DIR));
+  entry=(DIR *) AllocateMemory(sizeof(DIR));
   if (entry != (DIR *) NULL)
     entry->hSearch=
       FindFirstFile(file_specification,&entry->Win32FindData);
@@ -456,7 +477,7 @@ DIR *opendir(char *path)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function readdir returns a pointer to a structure representing the
+%  Method readdir returns a pointer to a structure representing the
 %  directory entry at the current position in the directory stream to
 %  which entry refers.
 %
@@ -470,7 +491,7 @@ DIR *opendir(char *path)
 %
 %
 */
-struct dirent *readdir(DIR *entry)
+Export struct dirent *readdir(DIR *entry)
 {
   int
     status;
@@ -497,12 +518,12 @@ struct dirent *readdir(DIR *entry)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%   Function seekdir sets the position of the next readdir() operation
+%   Method seekdir sets the position of the next readdir() operation
 %   on the directory stream.
 %
-%  The format of the seekdir routine is:
+%  The format of the seekdir method is:
 %
-%      seekdir(entry,position)
+%      void seekdir(DIR *entry,long position)
 %
 %  A description of each parameter follows:
 %
@@ -514,7 +535,7 @@ struct dirent *readdir(DIR *entry)
 %
 %
 */
-void seekdir(DIR *entry,long position)
+Export void seekdir(DIR *entry,long position)
 {
   assert(entry != (DIR *) NULL);
 }
@@ -530,12 +551,12 @@ void seekdir(DIR *entry,long position)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%   Function telldir returns the current location associated  with  the
+%   Method telldir returns the current location associated  with  the
 %   named directory stream.
 %
-%  The format of the telldir routine is:
+%  The format of the telldir method is:
 %
-%      telldir(entry)
+%      long telldir(DIR *entry)
 %
 %  A description of each parameter follows:
 %
@@ -543,7 +564,7 @@ void seekdir(DIR *entry,long position)
 %
 %
 */
-long telldir(DIR *entry)
+Export long telldir(DIR *entry)
 {
   assert(entry != (DIR *) NULL);
   return(0);
@@ -560,20 +581,26 @@ long telldir(DIR *entry)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function Dllmain is the entry point for the ImageMagick DLL library.
+%  Method Dllmain is the entry point for the ImageMagick DLL library.
 %
 %
 */
-BOOL WINAPI DllMain(HINSTANCE hInst,DWORD wDataSeg,LPVOID lpReserved)
+BOOL WINAPI DllMain(HINSTANCE instance,DWORD data_segment,LPVOID reserved)
 {
-  switch(wDataSeg)
+  switch(data_segment)
   {
-     case DLL_PROCESS_ATTACH:
-       return(1);
-     case DLL_PROCESS_DETACH:
-       break;
-     default:
-       return(1);
+    case DLL_PROCESS_ATTACH:
+    {
+      return(1);
+    }
+    case DLL_PROCESS_DETACH:
+    {
+      break;
+    }
+    default:
+    {
+      return(1);
+    }
   }
   return(0);
 }
