@@ -19,7 +19,7 @@
 %                             October 1992                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1998 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -85,7 +85,7 @@
 #include "regular.h"
 #include <termios.h>
 #include <fcntl.h>
-#if defined(SVR4) && !defined(linux)
+#if defined(HAVE_PTMX)
 #include <stropts.h>
 #include <sys/socket.h>
 #endif
@@ -167,7 +167,7 @@ static void DirectoryRequest(char *fileinfo,char *filename)
 
   status=0;
   for (p=filename; *p != '\0'; p++)
-    if (!isprint(*p))
+    if (!isprint((int) *p))
       *p=' ';
   if (*fileinfo == '\0')
     (void) fprintf(stdout,"%s\n",filename);
@@ -346,10 +346,19 @@ static void ExecuteFtp(char *hostname,char *port)
   /*
     Condition slave tty line.
   */
-#if defined(SVR4) && !defined(linux)
+#if defined(HAVE_PTMX)
   (void) ioctl(slave,I_PUSH,"ptem");
   (void) ioctl(slave,I_PUSH,"ldterm");
   (void) ioctl(slave,I_PUSH,"ttcompat");
+#endif
+#if defined(HAVE_PTMX_BSD)
+  {
+    char
+      buffer[10240];
+
+    if ((void) ioctl(slave,I_LOOK,buffer) != 0)
+      (void) ioctl(slave,I_PUSH,"ldterm");
+  }
 #endif
   (void) tcgetattr(slave,&attributes);
   attributes.c_iflag&=(~(BRKINT | IGNPAR | PARMRK | INPCK | ISTRIP | INLCR |
@@ -370,7 +379,7 @@ static void ExecuteFtp(char *hostname,char *port)
   (void) dup2(slave,STDOUT_FILENO);
   (void) dup2(slave,STDERR_FILENO);
   (void) close(slave);
-  (void) execlp("ftp","ftp","-n","-i","-g","-v",hostname,port,(char *) 0);
+  (void) execlp(XTP_FTP,"ftp","-n","-i","-g","-v",hostname,port,(char *) 0);
   perror("ftp");
   (void) kill(0,SIGTERM);
 }
@@ -525,7 +534,7 @@ static void GetPseudoTerminal(void)
   struct termios
     attributes;
 
-#if defined(SVR4) && !defined(linux)
+#if defined(HAVE_PTMX)
   *slave_tty='\0';
   master=open("/dev/ptmx",O_RDWR);
   if (master > 0)
@@ -626,7 +635,11 @@ static int MakeDirectory(char *directory)
   if ((p == (char *) NULL) || (p == directory))
     return(False);
   *p='\0';
+#if !defined(__OPENNT)
   if (lstat(directory,&info) < 0)
+#else
+  if (stat(directory,&info) < 0)
+#endif
     {
       /*
         Path component does not exist;  create it.
@@ -700,12 +713,14 @@ static void PrintRequest(char *filename,unsigned int verbose)
       (void) strcat(command," -\n");
   (void) write(master,command,strlen(command));
   (void) fprintf(stdout,"%s:\n",filename);
-  while (response=Wait())
+  while ((response=Wait()))
+  {
     if (status == 0)
       (void) fprintf(stdout,"%s\n",response);
     else
       if ((status == 5) || verbose)
         (void) fprintf(stderr,"%s\n",response);
+  }
 }
 
 /*
@@ -771,66 +786,60 @@ static void ProcessRequest(char *system_type,unsigned int prune,
     maximum_files,
     number_files;
 
-  if (*system_type == '\0')
-    {
-      /*
-        Determine system type.
-      */
-      (void) strcpy(system_type,"UNIX");
-      (void) strcpy(command,"quote syst\n");
-      (void) write(master,command,strlen(command));
-      while (response=Wait())
-        if (status == 2)
-          (void) strcpy(system_type,response+4);
-    }
   /*
     Ask remote server for a file listing.
   */
   (void) strcpy(command,"dir\n");
   if (!prune)
-    if (strncmp("VMS",system_type,3) == 0)
-      (void) strcpy(command,"ls [...]\n");
-    else
-      if ((strncmp("UNIX",system_type,4) == 0) ||
-          (strncmp("Windows_NT",system_type,10) == 0))
-        {
-          RegularExpression
-            *list_expression;
+    {
+      if (strncmp("VMS",system_type,3) == 0)
+        (void) strcpy(command,"ls [...]\n");
+      else
+        if ((strncmp("UNIX",system_type,4) == 0) ||
+            (strncmp("Windows_NT",system_type,10) == 0))
+          {
+            RegularExpression
+              *list_expression;
 
-          /*
-            Get a recursive file listing.
-          */
-          (void) write(master,command,strlen(command));
-          (void) strcpy(command,"ls -ltR\n");
-          list_expression=CompileRegularExpression(ListExpression);
-          while (response=Wait())
-            if ((status == 0) && (*response != '\0'))
-              if (ExecuteRegularExpression(list_expression,response))
-                {
-                  /*
-                    Remote site has a directory listing file.
-                  */
-                  (void) strncpy(filename,list_expression->subpattern[0],
-                    list_expression->pattern_length);
-                  (void) fprintf(stderr,"Using remote file listing %s...\n",
-                    filename);
-                  (void) sprintf(command,"get %s",filename);
-                  if (strcmp(filename+strlen(filename)-2,".Z") == 0)
-                    (void) strcat(command," |zcat\n");
-                  else
-                    if (strcmp(filename+strlen(filename)-3,".gz") == 0)
-                      (void) strcat(command," |gunzip -c\n");
+            /*
+              Get a recursive file listing.
+            */
+            (void) write(master,command,strlen(command));
+            (void) strcpy(command,"ls -ltR\n");
+            list_expression=CompileRegularExpression(ListExpression);
+            while ((response=Wait()))
+            {
+              if ((status == 0) && (*response != '\0'))
+                if (ExecuteRegularExpression(list_expression,response))
+                  {
+                    /*
+                      Remote site has a directory listing file.
+                    */
+                    (void) strncpy(filename,list_expression->subpattern[0],
+                      list_expression->pattern_length);
+                    (void) fprintf(stderr,"Using remote file listing %s...\n",
+                      filename);
+                    (void) sprintf(command,"get %s",filename);
+                    if (strcmp(filename+strlen(filename)-2,".Z") == 0)
+                      (void) strcat(command," |zcat\n");
                     else
-                      (void) strcat(command," -\n");
-                  while (Wait());
-                  break;
-                }
-          free((char *) list_expression);
-        }
+                      if (strcmp(filename+strlen(filename)-3,".gz") == 0)
+                        (void) strcat(command," |gunzip -c\n");
+                      else
+                        (void) strcat(command," -\n");
+                    while (Wait());
+                    break;
+                  }
+            }
+            free((char *) list_expression);
+          }
+    }
   (void) write(master,command,strlen(command));
-  while (response=Wait())
+  while ((response=Wait()))
+  {
     if ((status == 0) || (status == 5))
       break;
+  }
   if (status == 5)
     {
       /*
@@ -839,9 +848,11 @@ static void ProcessRequest(char *system_type,unsigned int prune,
       while (Wait());
       (void) strcpy(command,"dir\n");
       (void) write(master,command,strlen(command));
-      while (response=Wait())
+      while ((response=Wait()))
+      {
         if ((status == 0) || (status == 5))
           break;
+      }
     }
   status=(-1);
   if (response == (char *) NULL)
@@ -944,7 +955,7 @@ static void ProcessRequest(char *system_type,unsigned int prune,
       Error("Unable to allocate memory",(char *) NULL);
     (void) strcpy(filelist[number_files],filename);
     number_files++;
-  } while (response=Wait());
+  } while ((response=Wait()));
   free((char *) date_expression);
   free((char *) access_expression);
   if (!print_expression && !retrieve_expression)
@@ -957,7 +968,7 @@ static void ProcessRequest(char *system_type,unsigned int prune,
   /*
     Print, or retrieve a file.
   */
-  for (i=0; i < number_files; i++)
+  for (i=0; i < (int) number_files; i++)
   {
     if (print_expression)
       if (ExecuteRegularExpression(print_expression,filelist[i]))
@@ -1009,9 +1020,11 @@ static void RetrieveRequest(char *filename,unsigned int verbose)
   (void) MakeDirectory(filename);
   (void) sprintf(command,"get %s\n",filename);
   (void) write(master,command,strlen(command));
-  while (response=Wait())
+  while ((response=Wait()))
+  {
     if (verbose)
       (void) fprintf(stderr,"%s\n",response);
+  }
 }
 
 /*
@@ -1214,7 +1227,7 @@ static char *Wait(void)
           return((char *) NULL);
   } while (p < (line+sizeof(line)));
   *p='\0';
-  if (isdigit(*line))
+  if (isdigit((int) *line))
     status=atoi(line)/100;
   return(line);
 }
@@ -1286,6 +1299,9 @@ int main(int argc,char **argv)
   client_name=argv[0];
   if (argc < 2)
     Usage();
+#if defined(SOCKS)
+  SOCKSinit(argv[0]);
+#endif
   /*
     Initialize program variables.
   */
@@ -1378,7 +1394,7 @@ int main(int argc,char **argv)
   /*
     Parse command line arguments.
   */
-  for (*argv++; *argv && ((**argv == '-') || (**argv == '+')); argv++)
+  for (argv++; *argv && ((**argv == '-') || (**argv == '+')); argv++)
     switch (argv[0][1])
     {
       case 'a':
@@ -1420,6 +1436,8 @@ int main(int argc,char **argv)
       case 'g':
       {
         get_expression=filename;
+        if (localname == (char *) NULL)
+          localname=filename;
         break;
       }
       case 'p':
@@ -1456,6 +1474,8 @@ int main(int argc,char **argv)
         if (strncmp("put",*argv+1,2) == 0)
           {
             put_expression=filename;
+            if (remotename == (char *) NULL)
+              remotename=filename;
             break;
           }
         Error("Unrecognized option",*argv);
@@ -1490,18 +1510,23 @@ int main(int argc,char **argv)
       (put_expression == (char *) NULL))
     get_expression=filename;
   if (*protocol != '\0')
-    if (strcmp(protocol,"ftp:") != 0)
-      if ((get_expression == (char *) NULL) || (strlen(get_expression) == 0))
-        Error("Unsupported protocol",protocol);
-      else
+    {
+      if (strcmp(protocol,"ftp:") != 0)
         {
-          /*
-            Use GET to handle non-ftp protocol but only for *get*.
-          */
-          (void) sprintf(command,"GET %s > %s",url,filename);
-          status=system(command);
-          return(status < 0);
+          if ((get_expression == (char *) NULL) ||
+              (strlen(get_expression) == 0))
+            Error("Unsupported protocol",protocol);
+          else
+            {
+              /*
+                Use GET to handle non-ftp protocol but only for *get*.
+              */
+              (void) sprintf(command,"GET %s > %s",url,filename);
+              status=system(command);
+              return(status < 0);
+            }
         }
+    }
   if ((*ident == '\0') && (*user == '\0'))
     {
       struct passwd
@@ -1525,9 +1550,11 @@ int main(int argc,char **argv)
           p=ident+strlen(ident);
           *p++='@';
           (void) gethostname(p,64);
+#if !defined(__OPENNT)
           p=ident+strlen(ident);
           *p++='.';
           (void) getdomainname(p,64);
+#endif
         }
     }
   if (proxy != (char *) NULL)
@@ -1543,10 +1570,12 @@ int main(int argc,char **argv)
   if (host_info == (char *) NULL)
     Error("Unknown host",hostname);
   if (verbose)
-    if (*directory == '\0')
-      (void) fprintf(stderr,"%s\n",host_info);
-    else
-      (void) fprintf(stderr,"%s %s\n",host_info,directory);
+    {
+      if (*directory == '\0')
+        (void) fprintf(stderr,"%s\n",host_info);
+      else
+        (void) fprintf(stderr,"%s %s\n",host_info,directory);
+    }
   GetPseudoTerminal();
   /*
     Set signal handlers.
@@ -1568,7 +1597,7 @@ int main(int argc,char **argv)
   if (child == 0)
     ExecuteFtp(hostname,port);
   type_expression=CompileRegularExpression(TypeExpression);
-  while (response=Wait())
+  while ((response=Wait()))
   {
     if (verbose)
       (void) fprintf(stderr,"%s\n",response);
@@ -1588,7 +1617,7 @@ int main(int argc,char **argv)
   account_expression=CompileRegularExpression(AccountExpression);
   connect_expression=CompileRegularExpression(ConnectExpression);
   ident_expression=CompileRegularExpression(IdentExpression);
-  while (response=Wait())
+  while ((response=Wait()))
   {
     if (verbose)
       (void) fprintf(stderr,"%s\n",response);
@@ -1622,6 +1651,20 @@ int main(int argc,char **argv)
   free((char *) account_expression);
   if (timeout != 0)
     (void) alarm(timeout);  /* enable session timer. */
+  if (*system_type == '\0')
+    {
+      /*
+        Determine system type.
+      */
+      (void) strcpy(system_type,"UNIX");
+      (void) strcpy(command,"quote syst\n");
+      (void) write(master,command,strlen(command));
+      while ((response=Wait()))
+      {
+        if (status == 2)
+          (void) strcpy(system_type,response+4);
+      }
+    }
   if (*directory != '\0')
     {
       /*
@@ -1629,7 +1672,7 @@ int main(int argc,char **argv)
       */
       (void) sprintf(command,"cd %s\n",directory);
       (void) write(master,command,strlen(command));
-      while (response=Wait())
+      while ((response=Wait()))
       {
         if (verbose)
           (void) fprintf(stderr,"%s\n",response);
@@ -1638,9 +1681,11 @@ int main(int argc,char **argv)
       }
       (void) strcpy(command,"pwd\n");
       (void) write(master,command,strlen(command));
-      while (response=Wait())
+      while ((response=Wait()))
+      {
         if (verbose)
           (void) fprintf(stderr,"%s\n",response);
+      }
     }
   if (binary)
     {
@@ -1649,14 +1694,18 @@ int main(int argc,char **argv)
       */
       (void) strcpy(command,"binary\n");
       (void) write(master,command,strlen(command));
-      while (response=Wait())
+      while ((response=Wait()))
+      {
         if (verbose)
           (void) fprintf(stderr,"%s\n",response);
+      }
       (void) strcpy(command,"type\n");
       (void) write(master,command,strlen(command));
-      while (response=Wait())
+      while ((response=Wait()))
+      {
         if (verbose)
           (void) fprintf(stderr,"%s\n",response);
+      }
     }
   if (get_expression != (char *) NULL)
     {
@@ -1665,11 +1714,35 @@ int main(int argc,char **argv)
       */
       (void) sprintf(command,"mget %s\n",get_expression);
       if (!IsGlob(get_expression) && (localname != (char *) NULL))
-        (void) sprintf(command,"get %s %s\n",get_expression,localname);
+        {
+          if ((strncmp("UNIX",system_type,4) != 0) &&
+              (strncmp("Windows_NT",system_type,10) != 0))
+            (void) sprintf(command,"get %s %s\n",get_expression,localname);
+          else
+            (void) sprintf(command,"get %s %s~\n",get_expression,localname);
+        }
       (void) write(master,command,strlen(command));
-      while (response=Wait())
-        if ((status == 5) || verbose)
+      while ((response=Wait()))
+      {
+        if (verbose)
           (void) fprintf(stderr,"%s\n",response);
+        if (status == 5)
+          Error(response,user);
+      }
+      if (!IsGlob(get_expression) && (localname != (char *) NULL))
+        if ((strncmp("UNIX",system_type,4) == 0) ||
+            (strncmp("Windows_NT",system_type,10) == 0))
+          {
+            (void) sprintf(command,"rename %s~ %s\n",localname,localname);
+            (void) write(master,command,strlen(command));
+            while ((response=Wait()))
+            {
+              if (verbose)
+                (void) fprintf(stderr,"%s\n",response);
+              if (status == 5)
+                Error(response,user);
+            }
+          }
     }
   else
     if (put_expression != (char *) NULL)
@@ -1679,16 +1752,45 @@ int main(int argc,char **argv)
         */
         (void) strcpy(command,"glob on\n");
         (void) write(master,command,strlen(command));
-        while (response=Wait())
+        while ((response=Wait()))
+        {
           if (verbose)
             (void) fprintf(stderr,"%s\n",response);
+        }
         (void) sprintf(command,"mput %s\n",put_expression);
         if (!IsGlob(put_expression) && (remotename != (char *) NULL))
-          (void) sprintf(command,"put %s %s\n",put_expression,remotename);
+          {
+            if ((strncmp("UNIX",system_type,4) != 0) &&
+                (strncmp("Windows_NT",system_type,10) != 0))
+              (void) sprintf(command,"put %s %s\n",put_expression,remotename);
+            else
+              (void) sprintf(command,"put %s %s~\n",put_expression,remotename);
+          }
         (void) write(master,command,strlen(command));
-        while (response=Wait())
-          if ((status == 5) || verbose)
+        while ((response=Wait()))
+        {
+          if (verbose)
             (void) fprintf(stderr,"%s\n",response);
+          if (status == 5)
+            Error(response,user);
+        }
+        if (!IsGlob(put_expression) && (remotename != (char *) NULL))
+          {
+            if ((strncmp("UNIX",system_type,4) == 0) ||
+                (strncmp("Windows_NT",system_type,10) == 0))
+              {
+                (void) sprintf(command,"rename %s~ %s\n",remotename,
+                  remotename);
+                (void) write(master,command,strlen(command));
+                while ((response=Wait()))
+                {
+                  if (verbose)
+                    (void) fprintf(stderr,"%s\n",response);
+                  if (status == 5)
+                    Error(response,user);
+                }
+              }
+          }
       }
     else
       ProcessRequest(system_type,prune,verbose);
