@@ -1,10 +1,14 @@
 /*
   Include declarations
 */
+
 #if defined(__hpux)
 #define _HPUX_SOURCE  1
 #endif
 #include <stdio.h>
+#if defined(SOCKS)
+#include <socks.h>
+#endif
 #include <stdlib.h>
 #include <unistd.h>
 #include <assert.h>
@@ -24,6 +28,9 @@
 #include <stat.h>
 #endif
 #include <pwd.h>
+#if defined(HAVE_CONFIG_H)
+#include "config.h"
+#endif
 
 /*
   Define declarations for the xtp program.
@@ -47,8 +54,18 @@
     (void) fprintf(stderr," (%s)",qualifier);  \
   (void) fprintf(stderr,".\n");  \
 }
+#if defined(SVR4) && !defined(linux)
+#define HAVE_PTMX
+#endif
+#if !defined(XTP_FTP)
+#if defined(SOCKS)
+#define XTP_FTP "rftp"
+#else
+#define XTP_FTP "ftp"
+#endif
+#endif
 
 #ifndef lint
 static char
-  Version[]="@(#)ImageMagick 3.9.2 97/11/15 cristy@sympatico.org";
+  Version[]="@(#)ImageMagick 4.2.8 99/08/01 cristy@mystic.es.dupont.com";
 #endif
